from typing import List, Optional
from urllib.parse import quote_plus

from stripe._api_resource import APIResource


# TODO(major): 1704. Remove this. It is no longer used except for "nested_resource_url" and "nested_resource_request",
# which are unnecessary and deprecated and should also be removed.
def nested_resource_class_methods(
    resource: str,
    path: Optional[str] = None,
    operations: Optional[List[str]] = None,
    resource_plural: Optional[str] = None,
):
    if resource_plural is None:
        resource_plural = "%ss" % resource
    if path is None:
        path = resource_plural

    def wrapper(cls):
        def nested_resource_url(cls, id, nested_id=None):
            url = "%s/%s/%s" % (
                cls.class_url(),
                quote_plus(id),
                quote_plus(path),
            )
            if nested_id is not None:
                url += "/%s" % quote_plus(nested_id)
            return url

        resource_url_method = "%ss_url" % resource
        setattr(cls, resource_url_method, classmethod(nested_resource_url))

        def nested_resource_request(cls, method, url, **params):
            return APIResource._static_request(
                method,
                url,
                params=params,
            )

        resource_request_method = "%ss_request" % resource
        setattr(
            cls, resource_request_method, classmethod(nested_resource_request)
        )

        if operations is None:
            return cls

        for operation in operations:
            if operation == "create":

                def create_nested_resource(cls, id, **params):
                    url = getattr(cls, resource_url_method)(id)
                    return getattr(cls, resource_request_method)(
                        "post", url, **params
                    )

                create_method = "create_%s" % resource
                setattr(
                    cls, create_method, classmethod(create_nested_resource)
                )

            elif operation == "retrieve":

                def retrieve_nested_resource(cls, id, nested_id, **params):
                    url = getattr(cls, resource_url_method)(id, nested_id)
                    return getattr(cls, resource_request_method)(
                        "get", url, **params
                    )

                retrieve_method = "retrieve_%s" % resource
                setattr(
                    cls, retrieve_method, classmethod(retrieve_nested_resource)
                )

            elif operation == "update":

                def modify_nested_resource(cls, id, nested_id, **params):
                    url = getattr(cls, resource_url_method)(id, nested_id)
                    return getattr(cls, resource_request_method)(
                        "post", url, **params
                    )

                modify_method = "modify_%s" % resource
                setattr(
                    cls, modify_method, classmethod(modify_nested_resource)
                )

            elif operation == "delete":

                def delete_nested_resource(cls, id, nested_id, **params):
                    url = getattr(cls, resource_url_method)(id, nested_id)
                    return getattr(cls, resource_request_method)(
                        "delete", url, **params
                    )

                delete_method = "delete_%s" % resource
                setattr(
                    cls, delete_method, classmethod(delete_nested_resource)
                )

            elif operation == "list":

                def list_nested_resources(cls, id, **params):
                    url = getattr(cls, resource_url_method)(id)
                    return getattr(cls, resource_request_method)(
                        "get", url, **params
                    )

                list_method = "list_%s" % resource_plural
                setattr(cls, list_method, classmethod(list_nested_resources))

            else:
                raise ValueError("Unknown operation: %s" % operation)

        return cls

    return wrapper



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class MeterUpdateParams(TypedDict):
    display_name: NotRequired[str]
    """
    The meter's name. Not visible to the customer.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class SubscriptionModifyParams(RequestOptions):
    add_invoice_items: NotRequired[
        List["SubscriptionModifyParamsAddInvoiceItem"]
    ]
    """
    A list of prices and quantities that will generate invoice items appended to the next invoice for this subscription. You may pass up to 20 items.
    """
    application_fee_percent: NotRequired["Literal['']|float"]
    """
    A non-negative decimal between 0 and 100, with at most two decimal places. This represents the percentage of the subscription invoice total that will be transferred to the application owner's Stripe account. The request must be made by a platform account on a connected account in order to set an application fee percentage. For more information, see the application fees [documentation](https://stripe.com/docs/connect/subscriptions#collecting-fees-on-subscriptions).
    """
    automatic_tax: NotRequired["SubscriptionModifyParamsAutomaticTax"]
    """
    Automatic tax settings for this subscription. We recommend you only include this parameter when the existing value is being changed.
    """
    billing_cadence: NotRequired[str]
    """
    The Billing Cadence which controls the timing of recurring invoice generation for this subscription. If unset, the subscription will bill according to its own configured schedule and create its own invoices. If set, this subscription will be billed by the cadence instead, potentially sharing invoices with the other subscriptions linked to that Cadence.
    """
    billing_cycle_anchor: NotRequired[Literal["now", "unchanged"]]
    """
    Either `now` or `unchanged`. Setting the value to `now` resets the subscription's billing cycle anchor to the current time (in UTC). For more information, see the billing cycle [documentation](https://stripe.com/docs/billing/subscriptions/billing-cycle).
    """
    billing_schedules: NotRequired[
        "Literal['']|List[SubscriptionModifyParamsBillingSchedule]"
    ]
    """
    Sets the billing schedules for the subscription.
    """
    billing_thresholds: NotRequired[
        "Literal['']|SubscriptionModifyParamsBillingThresholds"
    ]
    """
    Define thresholds at which an invoice will be sent, and the subscription advanced to a new billing period. When updating, pass an empty string to remove previously-defined thresholds.
    """
    cancel_at: NotRequired[
        "Literal['']|int|Literal['max_period_end', 'min_period_end']"
    ]
    """
    A timestamp at which the subscription should cancel. If set to a date before the current period ends, this will cause a proration if prorations have been enabled using `proration_behavior`. If set during a future period, this will always cause a proration for that period.
    """
    cancel_at_period_end: NotRequired[bool]
    """
    Indicate whether this subscription should cancel at the end of the current period (`current_period_end`). Defaults to `false`.
    """
    cancellation_details: NotRequired[
        "SubscriptionModifyParamsCancellationDetails"
    ]
    """
    Details about why this subscription was cancelled
    """
    collection_method: NotRequired[
        Literal["charge_automatically", "send_invoice"]
    ]
    """
    Either `charge_automatically`, or `send_invoice`. When charging automatically, Stripe will attempt to pay this subscription at the end of the cycle using the default source attached to the customer. When sending an invoice, Stripe will email your customer an invoice with payment instructions and mark the subscription as `active`. Defaults to `charge_automatically`.
    """
    days_until_due: NotRequired[int]
    """
    Number of days a customer has to pay invoices generated by this subscription. Valid only for subscriptions where `collection_method` is set to `send_invoice`.
    """
    default_payment_method: NotRequired[str]
    """
    ID of the default payment method for the subscription. It must belong to the customer associated with the subscription. This takes precedence over `default_source`. If neither are set, invoices will use the customer's [invoice_settings.default_payment_method](https://stripe.com/docs/api/customers/object#customer_object-invoice_settings-default_payment_method) or [default_source](https://stripe.com/docs/api/customers/object#customer_object-default_source).
    """
    default_source: NotRequired["Literal['']|str"]
    """
    ID of the default payment source for the subscription. It must belong to the customer associated with the subscription and be in a chargeable state. If `default_payment_method` is also set, `default_payment_method` will take precedence. If neither are set, invoices will use the customer's [invoice_settings.default_payment_method](https://stripe.com/docs/api/customers/object#customer_object-invoice_settings-default_payment_method) or [default_source](https://stripe.com/docs/api/customers/object#customer_object-default_source).
    """
    default_tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates that will apply to any subscription item that does not have `tax_rates` set. Invoices created will have their `default_tax_rates` populated from the subscription. Pass an empty string to remove previously-defined tax rates.
    """
    description: NotRequired["Literal['']|str"]
    """
    The subscription's description, meant to be displayable to the customer. Use this field to optionally store an explanation of the subscription for rendering in Stripe surfaces and certain local payment methods UIs.
    """
    discounts: NotRequired[
        "Literal['']|List[SubscriptionModifyParamsDiscount]"
    ]
    """
    The coupons to redeem into discounts for the subscription. If not specified or empty, inherits the discount from the subscription's customer.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoice_settings: NotRequired["SubscriptionModifyParamsInvoiceSettings"]
    """
    All invoices will be billed using the specified settings.
    """
    items: NotRequired[List["SubscriptionModifyParamsItem"]]
    """
    A list of up to 20 subscription items, each with an attached price.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    off_session: NotRequired[bool]
    """
    Indicates if a customer is on or off-session while an invoice payment is attempted. Defaults to `false` (on-session).
    """
    on_behalf_of: NotRequired["Literal['']|str"]
    """
    The account on behalf of which to charge, for each of the subscription's invoices.
    """
    pause_collection: NotRequired[
        "Literal['']|SubscriptionModifyParamsPauseCollection"
    ]
    """
    If specified, payment collection for this subscription will be paused. Note that the subscription status will be unchanged and will not be updated to `paused`. Learn more about [pausing collection](https://stripe.com/docs/billing/subscriptions/pause-payment).
    """
    payment_behavior: NotRequired[
        Literal[
            "allow_incomplete",
            "default_incomplete",
            "error_if_incomplete",
            "pending_if_incomplete",
        ]
    ]
    """
    Use `allow_incomplete` to transition the subscription to `status=past_due` if a payment is required but cannot be paid. This allows you to manage scenarios where additional user actions are needed to pay a subscription's invoice. For example, SCA regulation may require 3DS authentication to complete payment. See the [SCA Migration Guide](https://stripe.com/docs/billing/migration/strong-customer-authentication) for Billing to learn more. This is the default behavior.

    Use `default_incomplete` to transition the subscription to `status=past_due` when payment is required and await explicit confirmation of the invoice's payment intent. This allows simpler management of scenarios where additional user actions are needed to pay a subscription's invoice. Such as failed payments, [SCA regulation](https://stripe.com/docs/billing/migration/strong-customer-authentication), or collecting a mandate for a bank debit payment method.

    Use `pending_if_incomplete` to update the subscription using [pending updates](https://stripe.com/docs/billing/subscriptions/pending-updates). When you use `pending_if_incomplete` you can only pass the parameters [supported by pending updates](https://stripe.com/docs/billing/pending-updates-reference#supported-attributes).

    Use `error_if_incomplete` if you want Stripe to return an HTTP 402 status code if a subscription's invoice cannot be paid. For example, if a payment method requires 3DS authentication due to SCA regulation and further user action is needed, this parameter does not update the subscription and returns an error instead. This was the default behavior for API versions prior to 2019-03-14. See the [changelog](https://docs.stripe.com/changelog/2019-03-14) to learn more.
    """
    payment_settings: NotRequired["SubscriptionModifyParamsPaymentSettings"]
    """
    Payment settings to pass to invoices created by the subscription.
    """
    pending_invoice_item_interval: NotRequired[
        "Literal['']|SubscriptionModifyParamsPendingInvoiceItemInterval"
    ]
    """
    Specifies an interval for how often to bill for any pending invoice items. It is analogous to calling [Create an invoice](https://stripe.com/docs/api#create_invoice) for the given subscription at the specified interval.
    """
    prebilling: NotRequired["SubscriptionModifyParamsPrebilling"]
    """
    If specified, the invoicing for the given billing cycle iterations will be processed now.
    """
    proration_behavior: NotRequired[
        Literal["always_invoice", "create_prorations", "none"]
    ]
    """
    Determines how to handle [prorations](https://stripe.com/docs/billing/subscriptions/prorations) when the billing cycle changes (e.g., when switching plans, resetting `billing_cycle_anchor=now`, or starting a trial), or if an item's `quantity` changes. The default value is `create_prorations`.
    """
    proration_date: NotRequired[int]
    """
    If set, prorations will be calculated as though the subscription was updated at the given time. This can be used to apply exactly the same prorations that were previewed with the [create preview](https://stripe.com/docs/api/invoices/create_preview) endpoint. `proration_date` can also be used to implement custom proration logic, such as prorating by day instead of by second, by providing the time that you wish to use for proration calculations.
    """
    transfer_data: NotRequired[
        "Literal['']|SubscriptionModifyParamsTransferData"
    ]
    """
    If specified, the funds from the subscription's invoices will be transferred to the destination and the ID of the resulting transfers will be found on the resulting charges. This will be unset if you POST an empty value.
    """
    trial_end: NotRequired["Literal['now']|int"]
    """
    Unix timestamp representing the end of the trial period the customer will get before being charged for the first time. This will always overwrite any trials that might apply via a subscribed plan. If set, `trial_end` will override the default trial period of the plan the customer is being subscribed to. The `billing_cycle_anchor` will be updated to the `trial_end` value. The special value `now` can be provided to end the customer's trial immediately. Can be at most two years from `billing_cycle_anchor`.
    """
    trial_from_plan: NotRequired[bool]
    """
    Indicates if a plan's `trial_period_days` should be applied to the subscription. Setting `trial_end` per subscription is preferred, and this defaults to `false`. Setting this flag to `true` together with `trial_end` is not allowed. See [Using trial periods on subscriptions](https://stripe.com/docs/billing/subscriptions/trials) to learn more.
    """
    trial_settings: NotRequired["SubscriptionModifyParamsTrialSettings"]
    """
    Settings related to subscription trials.
    """


class SubscriptionModifyParamsAddInvoiceItem(TypedDict):
    discounts: NotRequired[
        List["SubscriptionModifyParamsAddInvoiceItemDiscount"]
    ]
    """
    The coupons to redeem into discounts for the item.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    period: NotRequired["SubscriptionModifyParamsAddInvoiceItemPeriod"]
    """
    The period associated with this invoice item. If not set, `period.start.type` defaults to `max_item_period_start` and `period.end.type` defaults to `min_item_period_end`.
    """
    price: NotRequired[str]
    """
    The ID of the price object. One of `price` or `price_data` is required.
    """
    price_data: NotRequired["SubscriptionModifyParamsAddInvoiceItemPriceData"]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline. One of `price` or `price_data` is required.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item. Defaults to 1.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates which apply to the item. When set, the `default_tax_rates` do not apply to this item.
    """


class SubscriptionModifyParamsAddInvoiceItemDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "SubscriptionModifyParamsAddInvoiceItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class SubscriptionModifyParamsAddInvoiceItemDiscountDiscountEnd(TypedDict):
    duration: NotRequired[
        "SubscriptionModifyParamsAddInvoiceItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class SubscriptionModifyParamsAddInvoiceItemDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class SubscriptionModifyParamsAddInvoiceItemPeriod(TypedDict):
    end: "SubscriptionModifyParamsAddInvoiceItemPeriodEnd"
    """
    End of the invoice item period.
    """
    start: "SubscriptionModifyParamsAddInvoiceItemPeriodStart"
    """
    Start of the invoice item period.
    """


class SubscriptionModifyParamsAddInvoiceItemPeriodEnd(TypedDict):
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the end of the invoice item period. Must be greater than or equal to `period.start`.
    """
    type: Literal["min_item_period_end", "timestamp"]
    """
    Select how to calculate the end of the invoice item period.
    """


class SubscriptionModifyParamsAddInvoiceItemPeriodStart(TypedDict):
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the start of the invoice item period. Must be less than or equal to `period.end`.
    """
    type: Literal["max_item_period_start", "now", "timestamp"]
    """
    Select how to calculate the start of the invoice item period.
    """


class SubscriptionModifyParamsAddInvoiceItemPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge or a negative integer representing the amount to credit to the customer.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class SubscriptionModifyParamsAutomaticTax(TypedDict):
    enabled: bool
    """
    Enabled automatic tax calculation which will automatically compute tax rates on all invoices generated by the subscription.
    """
    liability: NotRequired["SubscriptionModifyParamsAutomaticTaxLiability"]
    """
    The account that's liable for tax. If set, the business address and tax registrations required to perform the tax calculation are loaded from this account. The tax transaction is returned in the report of the connected account.
    """


class SubscriptionModifyParamsAutomaticTaxLiability(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class SubscriptionModifyParamsBillingSchedule(TypedDict):
    applies_to: NotRequired[
        List["SubscriptionModifyParamsBillingScheduleAppliesTo"]
    ]
    """
    Configure billing schedule differently for individual subscription items.
    """
    bill_until: NotRequired["SubscriptionModifyParamsBillingScheduleBillUntil"]
    """
    The end date for the billing schedule.
    """
    key: NotRequired[str]
    """
    Specify a key for the billing schedule. Must be unique to this field, alphanumeric, and up to 200 characters. If not provided, a unique key will be generated.
    """


class SubscriptionModifyParamsBillingScheduleAppliesTo(TypedDict):
    price: NotRequired[str]
    """
    The ID of the price object.
    """
    type: Literal["price"]
    """
    Controls which subscription items the billing schedule applies to.
    """


class SubscriptionModifyParamsBillingScheduleBillUntil(TypedDict):
    duration: NotRequired[
        "SubscriptionModifyParamsBillingScheduleBillUntilDuration"
    ]
    """
    Specifies the billing period.
    """
    timestamp: NotRequired[int]
    """
    The end date of the billing schedule.
    """
    type: Literal["duration", "timestamp"]
    """
    Describes how the billing schedule will determine the end date. Either `duration` or `timestamp`.
    """


class SubscriptionModifyParamsBillingScheduleBillUntilDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing duration. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The multiplier applied to the interval.
    """


class SubscriptionModifyParamsBillingThresholds(TypedDict):
    amount_gte: NotRequired[int]
    """
    Monetary threshold that triggers the subscription to advance to a new billing period
    """
    reset_billing_cycle_anchor: NotRequired[bool]
    """
    Indicates if the `billing_cycle_anchor` should be reset when a threshold is reached. If true, `billing_cycle_anchor` will be updated to the date/time the threshold was last reached; otherwise, the value will remain unchanged.
    """


class SubscriptionModifyParamsCancellationDetails(TypedDict):
    comment: NotRequired["Literal['']|str"]
    """
    Additional comments about why the user canceled the subscription, if the subscription was canceled explicitly by the user.
    """
    feedback: NotRequired[
        "Literal['']|Literal['customer_service', 'low_quality', 'missing_features', 'other', 'switched_service', 'too_complex', 'too_expensive', 'unused']"
    ]
    """
    The customer submitted reason for why they canceled, if the subscription was canceled explicitly by the user.
    """


class SubscriptionModifyParamsDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired["SubscriptionModifyParamsDiscountDiscountEnd"]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class SubscriptionModifyParamsDiscountDiscountEnd(TypedDict):
    duration: NotRequired[
        "SubscriptionModifyParamsDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class SubscriptionModifyParamsDiscountDiscountEndDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class SubscriptionModifyParamsInvoiceSettings(TypedDict):
    account_tax_ids: NotRequired["Literal['']|List[str]"]
    """
    The account tax IDs associated with the subscription. Will be set on invoices generated by the subscription.
    """
    issuer: NotRequired["SubscriptionModifyParamsInvoiceSettingsIssuer"]
    """
    The connected account that issues the invoice. The invoice is presented with the branding and support information of the specified account.
    """


class SubscriptionModifyParamsInvoiceSettingsIssuer(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class SubscriptionModifyParamsItem(TypedDict):
    billing_thresholds: NotRequired[
        "Literal['']|SubscriptionModifyParamsItemBillingThresholds"
    ]
    """
    Define thresholds at which an invoice will be sent, and the subscription advanced to a new billing period. Pass an empty string to remove previously-defined thresholds.
    """
    clear_usage: NotRequired[bool]
    """
    Delete all usage for a given subscription item. You must pass this when deleting a usage records subscription item. `clear_usage` has no effect if the plan has a billing meter attached.
    """
    deleted: NotRequired[bool]
    """
    A flag that, if set to `true`, will delete the specified item.
    """
    discounts: NotRequired[
        "Literal['']|List[SubscriptionModifyParamsItemDiscount]"
    ]
    """
    The coupons to redeem into discounts for the subscription item.
    """
    id: NotRequired[str]
    """
    Subscription item to update.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    plan: NotRequired[str]
    """
    Plan ID for this item, as a string.
    """
    price: NotRequired[str]
    """
    The ID of the price object. One of `price` or `price_data` is required. When changing a subscription item's price, `quantity` is set to 1 unless a `quantity` parameter is provided.
    """
    price_data: NotRequired["SubscriptionModifyParamsItemPriceData"]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline. One of `price` or `price_data` is required.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    A list of [Tax Rate](https://stripe.com/docs/api/tax_rates) ids. These Tax Rates will override the [`default_tax_rates`](https://stripe.com/docs/api/subscriptions/create#create_subscription-default_tax_rates) on the Subscription. When updating, pass an empty string to remove previously-defined tax rates.
    """
    current_trial: NotRequired["SubscriptionModifyParamsItemCurrentTrial"]
    """
    The trial offer to apply to this subscription item.
    """


class SubscriptionModifyParamsItemBillingThresholds(TypedDict):
    usage_gte: int
    """
    Number of units that meets the billing threshold to advance the subscription to a new billing period (e.g., it takes 10 $5 units to meet a $50 [monetary threshold](https://stripe.com/docs/api/subscriptions/update#update_subscription-billing_thresholds-amount_gte))
    """


class SubscriptionModifyParamsItemDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "SubscriptionModifyParamsItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class SubscriptionModifyParamsItemDiscountDiscountEnd(TypedDict):
    duration: NotRequired[
        "SubscriptionModifyParamsItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class SubscriptionModifyParamsItemDiscountDiscountEndDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class SubscriptionModifyParamsItemPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    recurring: "SubscriptionModifyParamsItemPriceDataRecurring"
    """
    The recurring components of a price such as `interval` and `interval_count`.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class SubscriptionModifyParamsItemPriceDataRecurring(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing frequency. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between subscription billings. For example, `interval=month` and `interval_count=3` bills every 3 months. Maximum of three years interval allowed (3 years, 36 months, or 156 weeks).
    """


class SubscriptionModifyParamsItemCurrentTrial(TypedDict):
    trial_end: NotRequired[int]
    """
    Unix timestamp representing the end of the trial offer period. Required when the trial offer has `duration.type=timestamp`. Cannot be specified when `duration.type=relative`.
    """
    trial_offer: str
    """
    The ID of the trial offer to apply to the subscription item.
    """


class SubscriptionModifyParamsPauseCollection(TypedDict):
    behavior: Literal["keep_as_draft", "mark_uncollectible", "void"]
    """
    The payment collection behavior for this subscription while paused. One of `keep_as_draft`, `mark_uncollectible`, or `void`.
    """
    resumes_at: NotRequired[int]
    """
    The time after which the subscription will resume collecting payments.
    """


class SubscriptionModifyParamsPaymentSettings(TypedDict):
    payment_method_options: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptions"
    ]
    """
    Payment-method-specific configuration to provide to invoices created by the subscription.
    """
    payment_method_types: NotRequired[
        "Literal['']|List[Literal['ach_credit_transfer', 'ach_debit', 'acss_debit', 'affirm', 'amazon_pay', 'au_becs_debit', 'bacs_debit', 'bancontact', 'boleto', 'card', 'cashapp', 'crypto', 'custom', 'customer_balance', 'eps', 'fpx', 'giropay', 'grabpay', 'id_bank_transfer', 'ideal', 'jp_credit_transfer', 'kakao_pay', 'klarna', 'konbini', 'kr_card', 'link', 'multibanco', 'naver_pay', 'nz_bank_account', 'p24', 'payco', 'paynow', 'paypal', 'pix', 'promptpay', 'revolut_pay', 'sepa_credit_transfer', 'sepa_debit', 'sofort', 'stripe_balance', 'swish', 'upi', 'us_bank_account', 'wechat_pay']]"
    ]
    """
    The list of payment method types (e.g. card) to provide to the invoice's PaymentIntent. If not set, Stripe attempts to automatically determine the types to use by looking at the invoice's default payment method, the subscription's default payment method, the customer's default payment method, and your [invoice template settings](https://dashboard.stripe.com/settings/billing/invoice). Should not be specified with payment_method_configuration
    """
    save_default_payment_method: NotRequired[Literal["off", "on_subscription"]]
    """
    Configure whether Stripe updates `subscription.default_payment_method` when payment succeeds. Defaults to `off` if unspecified.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptions(TypedDict):
    acss_debit: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebit"
    ]
    """
    This sub-hash contains details about the Canadian pre-authorized debit payment method options to pass to the invoice's PaymentIntent.
    """
    bancontact: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsBancontact"
    ]
    """
    This sub-hash contains details about the Bancontact payment method options to pass to the invoice's PaymentIntent.
    """
    card: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCard"
    ]
    """
    This sub-hash contains details about the Card payment method options to pass to the invoice's PaymentIntent.
    """
    customer_balance: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalance"
    ]
    """
    This sub-hash contains details about the Bank transfer payment method options to pass to the invoice's PaymentIntent.
    """
    id_bank_transfer: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsIdBankTransfer"
    ]
    """
    This sub-hash contains details about the Indonesia bank transfer payment method options to pass to the invoice's PaymentIntent.
    """
    konbini: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsKonbini"
    ]
    """
    This sub-hash contains details about the Konbini payment method options to pass to the invoice's PaymentIntent.
    """
    pix: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsPix"
    ]
    """
    This sub-hash contains details about the Pix payment method options to pass to the invoice's PaymentIntent.
    """
    sepa_debit: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsSepaDebit"
    ]
    """
    This sub-hash contains details about the SEPA Direct Debit payment method options to pass to the invoice's PaymentIntent.
    """
    upi: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUpi"
    ]
    """
    This sub-hash contains details about the UPI payment method options to pass to the invoice's PaymentIntent.
    """
    us_bank_account: NotRequired[
        "Literal['']|SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccount"
    ]
    """
    This sub-hash contains details about the ACH direct debit payment method options to pass to the invoice's PaymentIntent.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebit(
    TypedDict,
):
    mandate_options: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Verification method for the intent
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebitMandateOptions(
    TypedDict,
):
    transaction_type: NotRequired[Literal["business", "personal"]]
    """
    Transaction type of the mandate.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsBancontact(
    TypedDict,
):
    preferred_language: NotRequired[Literal["de", "en", "fr", "nl"]]
    """
    Preferred language of the Bancontact authorization page that the customer is redirected to.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCard(
    TypedDict,
):
    mandate_options: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCardMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate for cards issued in India.
    """
    network: NotRequired[
        Literal[
            "amex",
            "cartes_bancaires",
            "diners",
            "discover",
            "eftpos_au",
            "girocard",
            "interac",
            "jcb",
            "link",
            "mastercard",
            "unionpay",
            "unknown",
            "visa",
        ]
    ]
    """
    Selected network to process this Subscription on. Depends on the available networks of the card attached to the Subscription. Can be only set confirm-time.
    """
    request_three_d_secure: NotRequired[
        Literal["any", "automatic", "challenge"]
    ]
    """
    We strongly recommend that you rely on our SCA Engine to automatically prompt your customers for authentication based on risk level and [other requirements](https://stripe.com/docs/strong-customer-authentication). However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option. Read our guide on [manually requesting 3D Secure](https://stripe.com/docs/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCardMandateOptions(
    TypedDict,
):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalance(
    TypedDict,
):
    bank_transfer: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransfer"
    ]
    """
    Configuration for the bank transfer funding type, if the `funding_type` is set to `bank_transfer`.
    """
    funding_type: NotRequired[str]
    """
    The funding method type to be used when there are not enough funds in the customer balance. Permitted values include: `bank_transfer`.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransfer(
    TypedDict,
):
    eu_bank_transfer: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer"
    ]
    """
    Configuration for eu_bank_transfer funding type.
    """
    type: NotRequired[str]
    """
    The bank transfer type that can be used for funding. Permitted values include: `eu_bank_transfer`, `gb_bank_transfer`, `jp_bank_transfer`, `mx_bank_transfer`, or `us_bank_transfer`.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer(
    TypedDict,
):
    country: str
    """
    The desired country code of the bank account information. Permitted values include: `BE`, `DE`, `ES`, `FR`, `IE`, or `NL`.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsIdBankTransfer(
    TypedDict,
):
    pass


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsKonbini(
    TypedDict,
):
    pass


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsPix(
    TypedDict
):
    mandate_options: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsPixMandateOptions"
    ]
    """
    Configuration options for setting up a mandate
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsPixMandateOptions(
    TypedDict,
):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments. If not provided, defaults to 40000.
    """
    amount_includes_iof: NotRequired[Literal["always", "never"]]
    """
    Determines if the amount includes the IOF tax. Defaults to `never`.
    """
    end_date: NotRequired[str]
    """
    Date when the mandate expires and no further payments will be charged, in `YYYY-MM-DD`. If not provided, the mandate will be active until canceled.
    """
    payment_schedule: NotRequired[
        Literal["halfyearly", "monthly", "quarterly", "weekly", "yearly"]
    ]
    """
    Schedule at which the future payments will be charged. Defaults to `weekly`.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsSepaDebit(
    TypedDict,
):
    pass


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUpi(
    TypedDict
):
    mandate_options: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUpiMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUpiMandateOptions(
    TypedDict,
):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """
    end_date: NotRequired[int]
    """
    End date of the mandate or subscription. If not provided, the mandate will be active until canceled. If provided, end date should be after start date.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccount(
    TypedDict,
):
    financial_connections: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnections"
    ]
    """
    Additional fields for Financial Connections Session creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Verification method for the intent
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnections(
    TypedDict,
):
    filters: NotRequired[
        "SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters"
    ]
    """
    Provide filters for the linked accounts that the customer can select for the payment method.
    """
    permissions: NotRequired[
        List[
            Literal["balances", "ownership", "payment_method", "transactions"]
        ]
    ]
    """
    The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included. Valid permissions include: `balances`, `ownership`, `payment_method`, and `transactions`.
    """
    prefetch: NotRequired[
        List[
            Literal[
                "balances", "inferred_balances", "ownership", "transactions"
            ]
        ]
    ]
    """
    List of data features that you would like to retrieve upon account creation.
    """


class SubscriptionModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters(
    TypedDict,
):
    account_subcategories: NotRequired[List[Literal["checking", "savings"]]]
    """
    The account subcategories to use to filter for selectable accounts. Valid subcategories are `checking` and `savings`.
    """
    institution: NotRequired[str]
    """
    ID of the institution to use to filter for selectable accounts.
    """


class SubscriptionModifyParamsPendingInvoiceItemInterval(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies invoicing frequency. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between invoices. For example, `interval=month` and `interval_count=3` bills every 3 months. Maximum of one year interval allowed (1 year, 12 months, or 52 weeks).
    """


class SubscriptionModifyParamsPrebilling(TypedDict):
    iterations: int
    """
    This is used to determine the number of billing cycles to prebill.
    """
    update_behavior: NotRequired[Literal["prebill", "reset"]]
    """
    Whether to cancel or preserve `prebilling` if the subscription is updated during the prebilled period. The default value is `reset`.
    """


class SubscriptionModifyParamsTransferData(TypedDict):
    amount_percent: NotRequired[float]
    """
    A non-negative decimal between 0 and 100, with at most two decimal places. This represents the percentage of the subscription invoice total that will be transferred to the destination account. By default, the entire amount is transferred to the destination.
    """
    destination: str
    """
    ID of an existing, connected Stripe account.
    """


class SubscriptionModifyParamsTrialSettings(TypedDict):
    end_behavior: "SubscriptionModifyParamsTrialSettingsEndBehavior"
    """
    Defines how the subscription should behave when the user's free trial ends.
    """


class SubscriptionModifyParamsTrialSettingsEndBehavior(TypedDict):
    missing_payment_method: Literal["cancel", "create_invoice", "pause"]
    """
    Indicates how the subscription should change when the trial ends if the user did not provide a payment method.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing import Union
from typing_extensions import TYPE_CHECKING
from stripe.v2.core._event import UnknownEventNotification
from stripe._stripe_object import StripeObject

if TYPE_CHECKING:
    from stripe.events._v1_billing_meter_error_report_triggered_event import (
        V1BillingMeterErrorReportTriggeredEventNotification,
    )
    from stripe.events._v1_billing_meter_no_meter_found_event import (
        V1BillingMeterNoMeterFoundEventNotification,
    )
    from stripe.events._v2_billing_cadence_billed_event import (
        V2BillingCadenceBilledEventNotification,
    )
    from stripe.events._v2_billing_cadence_canceled_event import (
        V2BillingCadenceCanceledEventNotification,
    )
    from stripe.events._v2_billing_cadence_created_event import (
        V2BillingCadenceCreatedEventNotification,
    )
    from stripe.events._v2_billing_license_fee_created_event import (
        V2BillingLicenseFeeCreatedEventNotification,
    )
    from stripe.events._v2_billing_license_fee_updated_event import (
        V2BillingLicenseFeeUpdatedEventNotification,
    )
    from stripe.events._v2_billing_license_fee_version_created_event import (
        V2BillingLicenseFeeVersionCreatedEventNotification,
    )
    from stripe.events._v2_billing_licensed_item_created_event import (
        V2BillingLicensedItemCreatedEventNotification,
    )
    from stripe.events._v2_billing_licensed_item_updated_event import (
        V2BillingLicensedItemUpdatedEventNotification,
    )
    from stripe.events._v2_billing_metered_item_created_event import (
        V2BillingMeteredItemCreatedEventNotification,
    )
    from stripe.events._v2_billing_metered_item_updated_event import (
        V2BillingMeteredItemUpdatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_created_event import (
        V2BillingPricingPlanCreatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_updated_event import (
        V2BillingPricingPlanUpdatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_component_created_event import (
        V2BillingPricingPlanComponentCreatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_component_updated_event import (
        V2BillingPricingPlanComponentUpdatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_collection_awaiting_customer_action_event import (
        V2BillingPricingPlanSubscriptionCollectionAwaitingCustomerActionEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_collection_current_event import (
        V2BillingPricingPlanSubscriptionCollectionCurrentEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_collection_past_due_event import (
        V2BillingPricingPlanSubscriptionCollectionPastDueEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_collection_paused_event import (
        V2BillingPricingPlanSubscriptionCollectionPausedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_collection_unpaid_event import (
        V2BillingPricingPlanSubscriptionCollectionUnpaidEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_servicing_activated_event import (
        V2BillingPricingPlanSubscriptionServicingActivatedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_servicing_canceled_event import (
        V2BillingPricingPlanSubscriptionServicingCanceledEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_subscription_servicing_paused_event import (
        V2BillingPricingPlanSubscriptionServicingPausedEventNotification,
    )
    from stripe.events._v2_billing_pricing_plan_version_created_event import (
        V2BillingPricingPlanVersionCreatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_created_event import (
        V2BillingRateCardCreatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_updated_event import (
        V2BillingRateCardUpdatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_rate_created_event import (
        V2BillingRateCardRateCreatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_activated_event import (
        V2BillingRateCardSubscriptionActivatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_canceled_event import (
        V2BillingRateCardSubscriptionCanceledEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_collection_awaiting_customer_action_event import (
        V2BillingRateCardSubscriptionCollectionAwaitingCustomerActionEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_collection_current_event import (
        V2BillingRateCardSubscriptionCollectionCurrentEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_collection_past_due_event import (
        V2BillingRateCardSubscriptionCollectionPastDueEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_collection_paused_event import (
        V2BillingRateCardSubscriptionCollectionPausedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_collection_unpaid_event import (
        V2BillingRateCardSubscriptionCollectionUnpaidEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_servicing_activated_event import (
        V2BillingRateCardSubscriptionServicingActivatedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_servicing_canceled_event import (
        V2BillingRateCardSubscriptionServicingCanceledEventNotification,
    )
    from stripe.events._v2_billing_rate_card_subscription_servicing_paused_event import (
        V2BillingRateCardSubscriptionServicingPausedEventNotification,
    )
    from stripe.events._v2_billing_rate_card_version_created_event import (
        V2BillingRateCardVersionCreatedEventNotification,
    )
    from stripe.events._v2_core_account_closed_event import (
        V2CoreAccountClosedEventNotification,
    )
    from stripe.events._v2_core_account_created_event import (
        V2CoreAccountCreatedEventNotification,
    )
    from stripe.events._v2_core_account_updated_event import (
        V2CoreAccountUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_card_creator_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationCardCreatorCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_card_creator_updated_event import (
        V2CoreAccountIncludingConfigurationCardCreatorUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_customer_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_customer_updated_event import (
        V2CoreAccountIncludingConfigurationCustomerUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_merchant_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_merchant_updated_event import (
        V2CoreAccountIncludingConfigurationMerchantUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_recipient_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_recipient_updated_event import (
        V2CoreAccountIncludingConfigurationRecipientUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_storer_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationStorerCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_storer_updated_event import (
        V2CoreAccountIncludingConfigurationStorerUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_defaults_updated_event import (
        V2CoreAccountIncludingDefaultsUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_identity_updated_event import (
        V2CoreAccountIncludingIdentityUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_requirements_updated_event import (
        V2CoreAccountIncludingRequirementsUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_link_returned_event import (
        V2CoreAccountLinkReturnedEventNotification,
    )
    from stripe.events._v2_core_account_person_created_event import (
        V2CoreAccountPersonCreatedEventNotification,
    )
    from stripe.events._v2_core_account_person_deleted_event import (
        V2CoreAccountPersonDeletedEventNotification,
    )
    from stripe.events._v2_core_account_person_updated_event import (
        V2CoreAccountPersonUpdatedEventNotification,
    )
    from stripe.events._v2_core_claimable_sandbox_claimed_event import (
        V2CoreClaimableSandboxClaimedEventNotification,
    )
    from stripe.events._v2_core_claimable_sandbox_created_event import (
        V2CoreClaimableSandboxCreatedEventNotification,
    )
    from stripe.events._v2_core_claimable_sandbox_expired_event import (
        V2CoreClaimableSandboxExpiredEventNotification,
    )
    from stripe.events._v2_core_claimable_sandbox_expiring_event import (
        V2CoreClaimableSandboxExpiringEventNotification,
    )
    from stripe.events._v2_core_claimable_sandbox_sandbox_details_owner_account_updated_event import (
        V2CoreClaimableSandboxSandboxDetailsOwnerAccountUpdatedEventNotification,
    )
    from stripe.events._v2_core_event_destination_ping_event import (
        V2CoreEventDestinationPingEventNotification,
    )
    from stripe.events._v2_core_health_api_error_firing_event import (
        V2CoreHealthApiErrorFiringEventNotification,
    )
    from stripe.events._v2_core_health_api_error_resolved_event import (
        V2CoreHealthApiErrorResolvedEventNotification,
    )
    from stripe.events._v2_core_health_api_latency_firing_event import (
        V2CoreHealthApiLatencyFiringEventNotification,
    )
    from stripe.events._v2_core_health_api_latency_resolved_event import (
        V2CoreHealthApiLatencyResolvedEventNotification,
    )
    from stripe.events._v2_core_health_authorization_rate_drop_firing_event import (
        V2CoreHealthAuthorizationRateDropFiringEventNotification,
    )
    from stripe.events._v2_core_health_authorization_rate_drop_resolved_event import (
        V2CoreHealthAuthorizationRateDropResolvedEventNotification,
    )
    from stripe.events._v2_core_health_event_generation_failure_resolved_event import (
        V2CoreHealthEventGenerationFailureResolvedEventNotification,
    )
    from stripe.events._v2_core_health_fraud_rate_increased_event import (
        V2CoreHealthFraudRateIncreasedEventNotification,
    )
    from stripe.events._v2_core_health_issuing_authorization_request_errors_firing_event import (
        V2CoreHealthIssuingAuthorizationRequestErrorsFiringEventNotification,
    )
    from stripe.events._v2_core_health_issuing_authorization_request_errors_resolved_event import (
        V2CoreHealthIssuingAuthorizationRequestErrorsResolvedEventNotification,
    )
    from stripe.events._v2_core_health_issuing_authorization_request_timeout_firing_event import (
        V2CoreHealthIssuingAuthorizationRequestTimeoutFiringEventNotification,
    )
    from stripe.events._v2_core_health_issuing_authorization_request_timeout_resolved_event import (
        V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventNotification,
    )
    from stripe.events._v2_core_health_payment_method_error_firing_event import (
        V2CoreHealthPaymentMethodErrorFiringEventNotification,
    )
    from stripe.events._v2_core_health_payment_method_error_resolved_event import (
        V2CoreHealthPaymentMethodErrorResolvedEventNotification,
    )
    from stripe.events._v2_core_health_traffic_volume_drop_firing_event import (
        V2CoreHealthTrafficVolumeDropFiringEventNotification,
    )
    from stripe.events._v2_core_health_traffic_volume_drop_resolved_event import (
        V2CoreHealthTrafficVolumeDropResolvedEventNotification,
    )
    from stripe.events._v2_core_health_webhook_latency_firing_event import (
        V2CoreHealthWebhookLatencyFiringEventNotification,
    )
    from stripe.events._v2_core_health_webhook_latency_resolved_event import (
        V2CoreHealthWebhookLatencyResolvedEventNotification,
    )
    from stripe.events._v2_iam_api_key_created_event import (
        V2IamApiKeyCreatedEventNotification,
    )
    from stripe.events._v2_iam_api_key_default_secret_revealed_event import (
        V2IamApiKeyDefaultSecretRevealedEventNotification,
    )
    from stripe.events._v2_iam_api_key_expired_event import (
        V2IamApiKeyExpiredEventNotification,
    )
    from stripe.events._v2_iam_api_key_permissions_updated_event import (
        V2IamApiKeyPermissionsUpdatedEventNotification,
    )
    from stripe.events._v2_iam_api_key_rotated_event import (
        V2IamApiKeyRotatedEventNotification,
    )
    from stripe.events._v2_iam_api_key_updated_event import (
        V2IamApiKeyUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_adjustment_created_event import (
        V2MoneyManagementAdjustmentCreatedEventNotification,
    )
    from stripe.events._v2_money_management_financial_account_created_event import (
        V2MoneyManagementFinancialAccountCreatedEventNotification,
    )
    from stripe.events._v2_money_management_financial_account_updated_event import (
        V2MoneyManagementFinancialAccountUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_financial_address_activated_event import (
        V2MoneyManagementFinancialAddressActivatedEventNotification,
    )
    from stripe.events._v2_money_management_financial_address_failed_event import (
        V2MoneyManagementFinancialAddressFailedEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_available_event import (
        V2MoneyManagementInboundTransferAvailableEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_bank_debit_failed_event import (
        V2MoneyManagementInboundTransferBankDebitFailedEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_bank_debit_processing_event import (
        V2MoneyManagementInboundTransferBankDebitProcessingEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_bank_debit_queued_event import (
        V2MoneyManagementInboundTransferBankDebitQueuedEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_bank_debit_returned_event import (
        V2MoneyManagementInboundTransferBankDebitReturnedEventNotification,
    )
    from stripe.events._v2_money_management_inbound_transfer_bank_debit_succeeded_event import (
        V2MoneyManagementInboundTransferBankDebitSucceededEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_canceled_event import (
        V2MoneyManagementOutboundPaymentCanceledEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_created_event import (
        V2MoneyManagementOutboundPaymentCreatedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_failed_event import (
        V2MoneyManagementOutboundPaymentFailedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_posted_event import (
        V2MoneyManagementOutboundPaymentPostedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_returned_event import (
        V2MoneyManagementOutboundPaymentReturnedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_payment_updated_event import (
        V2MoneyManagementOutboundPaymentUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_canceled_event import (
        V2MoneyManagementOutboundTransferCanceledEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_created_event import (
        V2MoneyManagementOutboundTransferCreatedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_failed_event import (
        V2MoneyManagementOutboundTransferFailedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_posted_event import (
        V2MoneyManagementOutboundTransferPostedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_returned_event import (
        V2MoneyManagementOutboundTransferReturnedEventNotification,
    )
    from stripe.events._v2_money_management_outbound_transfer_updated_event import (
        V2MoneyManagementOutboundTransferUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_payout_method_updated_event import (
        V2MoneyManagementPayoutMethodUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_received_credit_available_event import (
        V2MoneyManagementReceivedCreditAvailableEventNotification,
    )
    from stripe.events._v2_money_management_received_credit_failed_event import (
        V2MoneyManagementReceivedCreditFailedEventNotification,
    )
    from stripe.events._v2_money_management_received_credit_returned_event import (
        V2MoneyManagementReceivedCreditReturnedEventNotification,
    )
    from stripe.events._v2_money_management_received_credit_succeeded_event import (
        V2MoneyManagementReceivedCreditSucceededEventNotification,
    )
    from stripe.events._v2_money_management_received_debit_canceled_event import (
        V2MoneyManagementReceivedDebitCanceledEventNotification,
    )
    from stripe.events._v2_money_management_received_debit_failed_event import (
        V2MoneyManagementReceivedDebitFailedEventNotification,
    )
    from stripe.events._v2_money_management_received_debit_pending_event import (
        V2MoneyManagementReceivedDebitPendingEventNotification,
    )
    from stripe.events._v2_money_management_received_debit_succeeded_event import (
        V2MoneyManagementReceivedDebitSucceededEventNotification,
    )
    from stripe.events._v2_money_management_received_debit_updated_event import (
        V2MoneyManagementReceivedDebitUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_recipient_verification_created_event import (
        V2MoneyManagementRecipientVerificationCreatedEventNotification,
    )
    from stripe.events._v2_money_management_recipient_verification_updated_event import (
        V2MoneyManagementRecipientVerificationUpdatedEventNotification,
    )
    from stripe.events._v2_money_management_transaction_created_event import (
        V2MoneyManagementTransactionCreatedEventNotification,
    )
    from stripe.events._v2_money_management_transaction_updated_event import (
        V2MoneyManagementTransactionUpdatedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_attempt_failed_event import (
        V2PaymentsOffSessionPaymentAttemptFailedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_attempt_started_event import (
        V2PaymentsOffSessionPaymentAttemptStartedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_authorization_attempt_failed_event import (
        V2PaymentsOffSessionPaymentAuthorizationAttemptFailedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_authorization_attempt_started_event import (
        V2PaymentsOffSessionPaymentAuthorizationAttemptStartedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_canceled_event import (
        V2PaymentsOffSessionPaymentCanceledEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_created_event import (
        V2PaymentsOffSessionPaymentCreatedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_failed_event import (
        V2PaymentsOffSessionPaymentFailedEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_requires_capture_event import (
        V2PaymentsOffSessionPaymentRequiresCaptureEventNotification,
    )
    from stripe.events._v2_payments_off_session_payment_succeeded_event import (
        V2PaymentsOffSessionPaymentSucceededEventNotification,
    )
    from stripe.events._v2_reporting_report_run_created_event import (
        V2ReportingReportRunCreatedEventNotification,
    )
    from stripe.events._v2_reporting_report_run_failed_event import (
        V2ReportingReportRunFailedEventNotification,
    )
    from stripe.events._v2_reporting_report_run_succeeded_event import (
        V2ReportingReportRunSucceededEventNotification,
    )
    from stripe.events._v2_reporting_report_run_updated_event import (
        V2ReportingReportRunUpdatedEventNotification,
    )


_V2_EVENT_CLASS_LOOKUP = {
    "v1.billing.meter.error_report_triggered": (
        "stripe.events._v1_billing_meter_error_report_triggered_event",
        "V1BillingMeterErrorReportTriggeredEvent",
    ),
    "v1.billing.meter.no_meter_found": (
        "stripe.events._v1_billing_meter_no_meter_found_event",
        "V1BillingMeterNoMeterFoundEvent",
    ),
    "v2.billing.cadence.billed": (
        "stripe.events._v2_billing_cadence_billed_event",
        "V2BillingCadenceBilledEvent",
    ),
    "v2.billing.cadence.canceled": (
        "stripe.events._v2_billing_cadence_canceled_event",
        "V2BillingCadenceCanceledEvent",
    ),
    "v2.billing.cadence.created": (
        "stripe.events._v2_billing_cadence_created_event",
        "V2BillingCadenceCreatedEvent",
    ),
    "v2.billing.licensed_item.created": (
        "stripe.events._v2_billing_licensed_item_created_event",
        "V2BillingLicensedItemCreatedEvent",
    ),
    "v2.billing.licensed_item.updated": (
        "stripe.events._v2_billing_licensed_item_updated_event",
        "V2BillingLicensedItemUpdatedEvent",
    ),
    "v2.billing.license_fee.created": (
        "stripe.events._v2_billing_license_fee_created_event",
        "V2BillingLicenseFeeCreatedEvent",
    ),
    "v2.billing.license_fee.updated": (
        "stripe.events._v2_billing_license_fee_updated_event",
        "V2BillingLicenseFeeUpdatedEvent",
    ),
    "v2.billing.license_fee_version.created": (
        "stripe.events._v2_billing_license_fee_version_created_event",
        "V2BillingLicenseFeeVersionCreatedEvent",
    ),
    "v2.billing.metered_item.created": (
        "stripe.events._v2_billing_metered_item_created_event",
        "V2BillingMeteredItemCreatedEvent",
    ),
    "v2.billing.metered_item.updated": (
        "stripe.events._v2_billing_metered_item_updated_event",
        "V2BillingMeteredItemUpdatedEvent",
    ),
    "v2.billing.pricing_plan_component.created": (
        "stripe.events._v2_billing_pricing_plan_component_created_event",
        "V2BillingPricingPlanComponentCreatedEvent",
    ),
    "v2.billing.pricing_plan_component.updated": (
        "stripe.events._v2_billing_pricing_plan_component_updated_event",
        "V2BillingPricingPlanComponentUpdatedEvent",
    ),
    "v2.billing.pricing_plan.created": (
        "stripe.events._v2_billing_pricing_plan_created_event",
        "V2BillingPricingPlanCreatedEvent",
    ),
    "v2.billing.pricing_plan_subscription.collection_awaiting_customer_action": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_awaiting_customer_action_event",
        "V2BillingPricingPlanSubscriptionCollectionAwaitingCustomerActionEvent",
    ),
    "v2.billing.pricing_plan_subscription.collection_current": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_current_event",
        "V2BillingPricingPlanSubscriptionCollectionCurrentEvent",
    ),
    "v2.billing.pricing_plan_subscription.collection_past_due": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_past_due_event",
        "V2BillingPricingPlanSubscriptionCollectionPastDueEvent",
    ),
    "v2.billing.pricing_plan_subscription.collection_paused": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_paused_event",
        "V2BillingPricingPlanSubscriptionCollectionPausedEvent",
    ),
    "v2.billing.pricing_plan_subscription.collection_unpaid": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_unpaid_event",
        "V2BillingPricingPlanSubscriptionCollectionUnpaidEvent",
    ),
    "v2.billing.pricing_plan_subscription.servicing_activated": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_activated_event",
        "V2BillingPricingPlanSubscriptionServicingActivatedEvent",
    ),
    "v2.billing.pricing_plan_subscription.servicing_canceled": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_canceled_event",
        "V2BillingPricingPlanSubscriptionServicingCanceledEvent",
    ),
    "v2.billing.pricing_plan_subscription.servicing_paused": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_paused_event",
        "V2BillingPricingPlanSubscriptionServicingPausedEvent",
    ),
    "v2.billing.pricing_plan.updated": (
        "stripe.events._v2_billing_pricing_plan_updated_event",
        "V2BillingPricingPlanUpdatedEvent",
    ),
    "v2.billing.pricing_plan_version.created": (
        "stripe.events._v2_billing_pricing_plan_version_created_event",
        "V2BillingPricingPlanVersionCreatedEvent",
    ),
    "v2.billing.rate_card.created": (
        "stripe.events._v2_billing_rate_card_created_event",
        "V2BillingRateCardCreatedEvent",
    ),
    "v2.billing.rate_card_rate.created": (
        "stripe.events._v2_billing_rate_card_rate_created_event",
        "V2BillingRateCardRateCreatedEvent",
    ),
    "v2.billing.rate_card_subscription.activated": (
        "stripe.events._v2_billing_rate_card_subscription_activated_event",
        "V2BillingRateCardSubscriptionActivatedEvent",
    ),
    "v2.billing.rate_card_subscription.canceled": (
        "stripe.events._v2_billing_rate_card_subscription_canceled_event",
        "V2BillingRateCardSubscriptionCanceledEvent",
    ),
    "v2.billing.rate_card_subscription.collection_awaiting_customer_action": (
        "stripe.events._v2_billing_rate_card_subscription_collection_awaiting_customer_action_event",
        "V2BillingRateCardSubscriptionCollectionAwaitingCustomerActionEvent",
    ),
    "v2.billing.rate_card_subscription.collection_current": (
        "stripe.events._v2_billing_rate_card_subscription_collection_current_event",
        "V2BillingRateCardSubscriptionCollectionCurrentEvent",
    ),
    "v2.billing.rate_card_subscription.collection_past_due": (
        "stripe.events._v2_billing_rate_card_subscription_collection_past_due_event",
        "V2BillingRateCardSubscriptionCollectionPastDueEvent",
    ),
    "v2.billing.rate_card_subscription.collection_paused": (
        "stripe.events._v2_billing_rate_card_subscription_collection_paused_event",
        "V2BillingRateCardSubscriptionCollectionPausedEvent",
    ),
    "v2.billing.rate_card_subscription.collection_unpaid": (
        "stripe.events._v2_billing_rate_card_subscription_collection_unpaid_event",
        "V2BillingRateCardSubscriptionCollectionUnpaidEvent",
    ),
    "v2.billing.rate_card_subscription.servicing_activated": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_activated_event",
        "V2BillingRateCardSubscriptionServicingActivatedEvent",
    ),
    "v2.billing.rate_card_subscription.servicing_canceled": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_canceled_event",
        "V2BillingRateCardSubscriptionServicingCanceledEvent",
    ),
    "v2.billing.rate_card_subscription.servicing_paused": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_paused_event",
        "V2BillingRateCardSubscriptionServicingPausedEvent",
    ),
    "v2.billing.rate_card.updated": (
        "stripe.events._v2_billing_rate_card_updated_event",
        "V2BillingRateCardUpdatedEvent",
    ),
    "v2.billing.rate_card_version.created": (
        "stripe.events._v2_billing_rate_card_version_created_event",
        "V2BillingRateCardVersionCreatedEvent",
    ),
    "v2.core.account.closed": (
        "stripe.events._v2_core_account_closed_event",
        "V2CoreAccountClosedEvent",
    ),
    "v2.core.account.created": (
        "stripe.events._v2_core_account_created_event",
        "V2CoreAccountCreatedEvent",
    ),
    "v2.core.account[configuration.card_creator].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_card_creator_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationCardCreatorCapabilityStatusUpdatedEvent",
    ),
    "v2.core.account[configuration.card_creator].updated": (
        "stripe.events._v2_core_account_including_configuration_card_creator_updated_event",
        "V2CoreAccountIncludingConfigurationCardCreatorUpdatedEvent",
    ),
    "v2.core.account[configuration.customer].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_customer_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEvent",
    ),
    "v2.core.account[configuration.customer].updated": (
        "stripe.events._v2_core_account_including_configuration_customer_updated_event",
        "V2CoreAccountIncludingConfigurationCustomerUpdatedEvent",
    ),
    "v2.core.account[configuration.merchant].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_merchant_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEvent",
    ),
    "v2.core.account[configuration.merchant].updated": (
        "stripe.events._v2_core_account_including_configuration_merchant_updated_event",
        "V2CoreAccountIncludingConfigurationMerchantUpdatedEvent",
    ),
    "v2.core.account[configuration.recipient].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_recipient_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEvent",
    ),
    "v2.core.account[configuration.recipient].updated": (
        "stripe.events._v2_core_account_including_configuration_recipient_updated_event",
        "V2CoreAccountIncludingConfigurationRecipientUpdatedEvent",
    ),
    "v2.core.account[configuration.storer].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_storer_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationStorerCapabilityStatusUpdatedEvent",
    ),
    "v2.core.account[configuration.storer].updated": (
        "stripe.events._v2_core_account_including_configuration_storer_updated_event",
        "V2CoreAccountIncludingConfigurationStorerUpdatedEvent",
    ),
    "v2.core.account[defaults].updated": (
        "stripe.events._v2_core_account_including_defaults_updated_event",
        "V2CoreAccountIncludingDefaultsUpdatedEvent",
    ),
    "v2.core.account[identity].updated": (
        "stripe.events._v2_core_account_including_identity_updated_event",
        "V2CoreAccountIncludingIdentityUpdatedEvent",
    ),
    "v2.core.account[requirements].updated": (
        "stripe.events._v2_core_account_including_requirements_updated_event",
        "V2CoreAccountIncludingRequirementsUpdatedEvent",
    ),
    "v2.core.account_link.returned": (
        "stripe.events._v2_core_account_link_returned_event",
        "V2CoreAccountLinkReturnedEvent",
    ),
    "v2.core.account_person.created": (
        "stripe.events._v2_core_account_person_created_event",
        "V2CoreAccountPersonCreatedEvent",
    ),
    "v2.core.account_person.deleted": (
        "stripe.events._v2_core_account_person_deleted_event",
        "V2CoreAccountPersonDeletedEvent",
    ),
    "v2.core.account_person.updated": (
        "stripe.events._v2_core_account_person_updated_event",
        "V2CoreAccountPersonUpdatedEvent",
    ),
    "v2.core.account.updated": (
        "stripe.events._v2_core_account_updated_event",
        "V2CoreAccountUpdatedEvent",
    ),
    "v2.core.claimable_sandbox.claimed": (
        "stripe.events._v2_core_claimable_sandbox_claimed_event",
        "V2CoreClaimableSandboxClaimedEvent",
    ),
    "v2.core.claimable_sandbox.created": (
        "stripe.events._v2_core_claimable_sandbox_created_event",
        "V2CoreClaimableSandboxCreatedEvent",
    ),
    "v2.core.claimable_sandbox.expired": (
        "stripe.events._v2_core_claimable_sandbox_expired_event",
        "V2CoreClaimableSandboxExpiredEvent",
    ),
    "v2.core.claimable_sandbox.expiring": (
        "stripe.events._v2_core_claimable_sandbox_expiring_event",
        "V2CoreClaimableSandboxExpiringEvent",
    ),
    "v2.core.claimable_sandbox.sandbox_details_owner_account_updated": (
        "stripe.events._v2_core_claimable_sandbox_sandbox_details_owner_account_updated_event",
        "V2CoreClaimableSandboxSandboxDetailsOwnerAccountUpdatedEvent",
    ),
    "v2.core.event_destination.ping": (
        "stripe.events._v2_core_event_destination_ping_event",
        "V2CoreEventDestinationPingEvent",
    ),
    "v2.core.health.api_error.firing": (
        "stripe.events._v2_core_health_api_error_firing_event",
        "V2CoreHealthApiErrorFiringEvent",
    ),
    "v2.core.health.api_error.resolved": (
        "stripe.events._v2_core_health_api_error_resolved_event",
        "V2CoreHealthApiErrorResolvedEvent",
    ),
    "v2.core.health.api_latency.firing": (
        "stripe.events._v2_core_health_api_latency_firing_event",
        "V2CoreHealthApiLatencyFiringEvent",
    ),
    "v2.core.health.api_latency.resolved": (
        "stripe.events._v2_core_health_api_latency_resolved_event",
        "V2CoreHealthApiLatencyResolvedEvent",
    ),
    "v2.core.health.authorization_rate_drop.firing": (
        "stripe.events._v2_core_health_authorization_rate_drop_firing_event",
        "V2CoreHealthAuthorizationRateDropFiringEvent",
    ),
    "v2.core.health.authorization_rate_drop.resolved": (
        "stripe.events._v2_core_health_authorization_rate_drop_resolved_event",
        "V2CoreHealthAuthorizationRateDropResolvedEvent",
    ),
    "v2.core.health.event_generation_failure.resolved": (
        "stripe.events._v2_core_health_event_generation_failure_resolved_event",
        "V2CoreHealthEventGenerationFailureResolvedEvent",
    ),
    "v2.core.health.fraud_rate.increased": (
        "stripe.events._v2_core_health_fraud_rate_increased_event",
        "V2CoreHealthFraudRateIncreasedEvent",
    ),
    "v2.core.health.issuing_authorization_request_errors.firing": (
        "stripe.events._v2_core_health_issuing_authorization_request_errors_firing_event",
        "V2CoreHealthIssuingAuthorizationRequestErrorsFiringEvent",
    ),
    "v2.core.health.issuing_authorization_request_errors.resolved": (
        "stripe.events._v2_core_health_issuing_authorization_request_errors_resolved_event",
        "V2CoreHealthIssuingAuthorizationRequestErrorsResolvedEvent",
    ),
    "v2.core.health.issuing_authorization_request_timeout.firing": (
        "stripe.events._v2_core_health_issuing_authorization_request_timeout_firing_event",
        "V2CoreHealthIssuingAuthorizationRequestTimeoutFiringEvent",
    ),
    "v2.core.health.issuing_authorization_request_timeout.resolved": (
        "stripe.events._v2_core_health_issuing_authorization_request_timeout_resolved_event",
        "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent",
    ),
    "v2.core.health.payment_method_error.firing": (
        "stripe.events._v2_core_health_payment_method_error_firing_event",
        "V2CoreHealthPaymentMethodErrorFiringEvent",
    ),
    "v2.core.health.payment_method_error.resolved": (
        "stripe.events._v2_core_health_payment_method_error_resolved_event",
        "V2CoreHealthPaymentMethodErrorResolvedEvent",
    ),
    "v2.core.health.traffic_volume_drop.firing": (
        "stripe.events._v2_core_health_traffic_volume_drop_firing_event",
        "V2CoreHealthTrafficVolumeDropFiringEvent",
    ),
    "v2.core.health.traffic_volume_drop.resolved": (
        "stripe.events._v2_core_health_traffic_volume_drop_resolved_event",
        "V2CoreHealthTrafficVolumeDropResolvedEvent",
    ),
    "v2.core.health.webhook_latency.firing": (
        "stripe.events._v2_core_health_webhook_latency_firing_event",
        "V2CoreHealthWebhookLatencyFiringEvent",
    ),
    "v2.core.health.webhook_latency.resolved": (
        "stripe.events._v2_core_health_webhook_latency_resolved_event",
        "V2CoreHealthWebhookLatencyResolvedEvent",
    ),
    "v2.iam.api_key.created": (
        "stripe.events._v2_iam_api_key_created_event",
        "V2IamApiKeyCreatedEvent",
    ),
    "v2.iam.api_key.default_secret_revealed": (
        "stripe.events._v2_iam_api_key_default_secret_revealed_event",
        "V2IamApiKeyDefaultSecretRevealedEvent",
    ),
    "v2.iam.api_key.expired": (
        "stripe.events._v2_iam_api_key_expired_event",
        "V2IamApiKeyExpiredEvent",
    ),
    "v2.iam.api_key.permissions_updated": (
        "stripe.events._v2_iam_api_key_permissions_updated_event",
        "V2IamApiKeyPermissionsUpdatedEvent",
    ),
    "v2.iam.api_key.rotated": (
        "stripe.events._v2_iam_api_key_rotated_event",
        "V2IamApiKeyRotatedEvent",
    ),
    "v2.iam.api_key.updated": (
        "stripe.events._v2_iam_api_key_updated_event",
        "V2IamApiKeyUpdatedEvent",
    ),
    "v2.money_management.adjustment.created": (
        "stripe.events._v2_money_management_adjustment_created_event",
        "V2MoneyManagementAdjustmentCreatedEvent",
    ),
    "v2.money_management.financial_account.created": (
        "stripe.events._v2_money_management_financial_account_created_event",
        "V2MoneyManagementFinancialAccountCreatedEvent",
    ),
    "v2.money_management.financial_account.updated": (
        "stripe.events._v2_money_management_financial_account_updated_event",
        "V2MoneyManagementFinancialAccountUpdatedEvent",
    ),
    "v2.money_management.financial_address.activated": (
        "stripe.events._v2_money_management_financial_address_activated_event",
        "V2MoneyManagementFinancialAddressActivatedEvent",
    ),
    "v2.money_management.financial_address.failed": (
        "stripe.events._v2_money_management_financial_address_failed_event",
        "V2MoneyManagementFinancialAddressFailedEvent",
    ),
    "v2.money_management.inbound_transfer.available": (
        "stripe.events._v2_money_management_inbound_transfer_available_event",
        "V2MoneyManagementInboundTransferAvailableEvent",
    ),
    "v2.money_management.inbound_transfer.bank_debit_failed": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_failed_event",
        "V2MoneyManagementInboundTransferBankDebitFailedEvent",
    ),
    "v2.money_management.inbound_transfer.bank_debit_processing": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_processing_event",
        "V2MoneyManagementInboundTransferBankDebitProcessingEvent",
    ),
    "v2.money_management.inbound_transfer.bank_debit_queued": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_queued_event",
        "V2MoneyManagementInboundTransferBankDebitQueuedEvent",
    ),
    "v2.money_management.inbound_transfer.bank_debit_returned": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_returned_event",
        "V2MoneyManagementInboundTransferBankDebitReturnedEvent",
    ),
    "v2.money_management.inbound_transfer.bank_debit_succeeded": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_succeeded_event",
        "V2MoneyManagementInboundTransferBankDebitSucceededEvent",
    ),
    "v2.money_management.outbound_payment.canceled": (
        "stripe.events._v2_money_management_outbound_payment_canceled_event",
        "V2MoneyManagementOutboundPaymentCanceledEvent",
    ),
    "v2.money_management.outbound_payment.created": (
        "stripe.events._v2_money_management_outbound_payment_created_event",
        "V2MoneyManagementOutboundPaymentCreatedEvent",
    ),
    "v2.money_management.outbound_payment.failed": (
        "stripe.events._v2_money_management_outbound_payment_failed_event",
        "V2MoneyManagementOutboundPaymentFailedEvent",
    ),
    "v2.money_management.outbound_payment.posted": (
        "stripe.events._v2_money_management_outbound_payment_posted_event",
        "V2MoneyManagementOutboundPaymentPostedEvent",
    ),
    "v2.money_management.outbound_payment.returned": (
        "stripe.events._v2_money_management_outbound_payment_returned_event",
        "V2MoneyManagementOutboundPaymentReturnedEvent",
    ),
    "v2.money_management.outbound_payment.updated": (
        "stripe.events._v2_money_management_outbound_payment_updated_event",
        "V2MoneyManagementOutboundPaymentUpdatedEvent",
    ),
    "v2.money_management.outbound_transfer.canceled": (
        "stripe.events._v2_money_management_outbound_transfer_canceled_event",
        "V2MoneyManagementOutboundTransferCanceledEvent",
    ),
    "v2.money_management.outbound_transfer.created": (
        "stripe.events._v2_money_management_outbound_transfer_created_event",
        "V2MoneyManagementOutboundTransferCreatedEvent",
    ),
    "v2.money_management.outbound_transfer.failed": (
        "stripe.events._v2_money_management_outbound_transfer_failed_event",
        "V2MoneyManagementOutboundTransferFailedEvent",
    ),
    "v2.money_management.outbound_transfer.posted": (
        "stripe.events._v2_money_management_outbound_transfer_posted_event",
        "V2MoneyManagementOutboundTransferPostedEvent",
    ),
    "v2.money_management.outbound_transfer.returned": (
        "stripe.events._v2_money_management_outbound_transfer_returned_event",
        "V2MoneyManagementOutboundTransferReturnedEvent",
    ),
    "v2.money_management.outbound_transfer.updated": (
        "stripe.events._v2_money_management_outbound_transfer_updated_event",
        "V2MoneyManagementOutboundTransferUpdatedEvent",
    ),
    "v2.money_management.payout_method.updated": (
        "stripe.events._v2_money_management_payout_method_updated_event",
        "V2MoneyManagementPayoutMethodUpdatedEvent",
    ),
    "v2.money_management.received_credit.available": (
        "stripe.events._v2_money_management_received_credit_available_event",
        "V2MoneyManagementReceivedCreditAvailableEvent",
    ),
    "v2.money_management.received_credit.failed": (
        "stripe.events._v2_money_management_received_credit_failed_event",
        "V2MoneyManagementReceivedCreditFailedEvent",
    ),
    "v2.money_management.received_credit.returned": (
        "stripe.events._v2_money_management_received_credit_returned_event",
        "V2MoneyManagementReceivedCreditReturnedEvent",
    ),
    "v2.money_management.received_credit.succeeded": (
        "stripe.events._v2_money_management_received_credit_succeeded_event",
        "V2MoneyManagementReceivedCreditSucceededEvent",
    ),
    "v2.money_management.received_debit.canceled": (
        "stripe.events._v2_money_management_received_debit_canceled_event",
        "V2MoneyManagementReceivedDebitCanceledEvent",
    ),
    "v2.money_management.received_debit.failed": (
        "stripe.events._v2_money_management_received_debit_failed_event",
        "V2MoneyManagementReceivedDebitFailedEvent",
    ),
    "v2.money_management.received_debit.pending": (
        "stripe.events._v2_money_management_received_debit_pending_event",
        "V2MoneyManagementReceivedDebitPendingEvent",
    ),
    "v2.money_management.received_debit.succeeded": (
        "stripe.events._v2_money_management_received_debit_succeeded_event",
        "V2MoneyManagementReceivedDebitSucceededEvent",
    ),
    "v2.money_management.received_debit.updated": (
        "stripe.events._v2_money_management_received_debit_updated_event",
        "V2MoneyManagementReceivedDebitUpdatedEvent",
    ),
    "v2.money_management.recipient_verification.created": (
        "stripe.events._v2_money_management_recipient_verification_created_event",
        "V2MoneyManagementRecipientVerificationCreatedEvent",
    ),
    "v2.money_management.recipient_verification.updated": (
        "stripe.events._v2_money_management_recipient_verification_updated_event",
        "V2MoneyManagementRecipientVerificationUpdatedEvent",
    ),
    "v2.money_management.transaction.created": (
        "stripe.events._v2_money_management_transaction_created_event",
        "V2MoneyManagementTransactionCreatedEvent",
    ),
    "v2.money_management.transaction.updated": (
        "stripe.events._v2_money_management_transaction_updated_event",
        "V2MoneyManagementTransactionUpdatedEvent",
    ),
    "v2.payments.off_session_payment.attempt_failed": (
        "stripe.events._v2_payments_off_session_payment_attempt_failed_event",
        "V2PaymentsOffSessionPaymentAttemptFailedEvent",
    ),
    "v2.payments.off_session_payment.attempt_started": (
        "stripe.events._v2_payments_off_session_payment_attempt_started_event",
        "V2PaymentsOffSessionPaymentAttemptStartedEvent",
    ),
    "v2.payments.off_session_payment.authorization_attempt_failed": (
        "stripe.events._v2_payments_off_session_payment_authorization_attempt_failed_event",
        "V2PaymentsOffSessionPaymentAuthorizationAttemptFailedEvent",
    ),
    "v2.payments.off_session_payment.authorization_attempt_started": (
        "stripe.events._v2_payments_off_session_payment_authorization_attempt_started_event",
        "V2PaymentsOffSessionPaymentAuthorizationAttemptStartedEvent",
    ),
    "v2.payments.off_session_payment.canceled": (
        "stripe.events._v2_payments_off_session_payment_canceled_event",
        "V2PaymentsOffSessionPaymentCanceledEvent",
    ),
    "v2.payments.off_session_payment.created": (
        "stripe.events._v2_payments_off_session_payment_created_event",
        "V2PaymentsOffSessionPaymentCreatedEvent",
    ),
    "v2.payments.off_session_payment.failed": (
        "stripe.events._v2_payments_off_session_payment_failed_event",
        "V2PaymentsOffSessionPaymentFailedEvent",
    ),
    "v2.payments.off_session_payment.requires_capture": (
        "stripe.events._v2_payments_off_session_payment_requires_capture_event",
        "V2PaymentsOffSessionPaymentRequiresCaptureEvent",
    ),
    "v2.payments.off_session_payment.succeeded": (
        "stripe.events._v2_payments_off_session_payment_succeeded_event",
        "V2PaymentsOffSessionPaymentSucceededEvent",
    ),
    "v2.reporting.report_run.created": (
        "stripe.events._v2_reporting_report_run_created_event",
        "V2ReportingReportRunCreatedEvent",
    ),
    "v2.reporting.report_run.failed": (
        "stripe.events._v2_reporting_report_run_failed_event",
        "V2ReportingReportRunFailedEvent",
    ),
    "v2.reporting.report_run.succeeded": (
        "stripe.events._v2_reporting_report_run_succeeded_event",
        "V2ReportingReportRunSucceededEvent",
    ),
    "v2.reporting.report_run.updated": (
        "stripe.events._v2_reporting_report_run_updated_event",
        "V2ReportingReportRunUpdatedEvent",
    ),
}


def get_v2_event_class(type_: str):
    if type_ not in _V2_EVENT_CLASS_LOOKUP:
        return StripeObject

    import_path, class_name = _V2_EVENT_CLASS_LOOKUP[type_]
    return getattr(
        import_module(import_path),
        class_name,
    )


_V2_EVENT_NOTIFICATION_CLASS_LOOKUP = {
    "v1.billing.meter.error_report_triggered": (
        "stripe.events._v1_billing_meter_error_report_triggered_event",
        "V1BillingMeterErrorReportTriggeredEventNotification",
    ),
    "v1.billing.meter.no_meter_found": (
        "stripe.events._v1_billing_meter_no_meter_found_event",
        "V1BillingMeterNoMeterFoundEventNotification",
    ),
    "v2.billing.cadence.billed": (
        "stripe.events._v2_billing_cadence_billed_event",
        "V2BillingCadenceBilledEventNotification",
    ),
    "v2.billing.cadence.canceled": (
        "stripe.events._v2_billing_cadence_canceled_event",
        "V2BillingCadenceCanceledEventNotification",
    ),
    "v2.billing.cadence.created": (
        "stripe.events._v2_billing_cadence_created_event",
        "V2BillingCadenceCreatedEventNotification",
    ),
    "v2.billing.licensed_item.created": (
        "stripe.events._v2_billing_licensed_item_created_event",
        "V2BillingLicensedItemCreatedEventNotification",
    ),
    "v2.billing.licensed_item.updated": (
        "stripe.events._v2_billing_licensed_item_updated_event",
        "V2BillingLicensedItemUpdatedEventNotification",
    ),
    "v2.billing.license_fee.created": (
        "stripe.events._v2_billing_license_fee_created_event",
        "V2BillingLicenseFeeCreatedEventNotification",
    ),
    "v2.billing.license_fee.updated": (
        "stripe.events._v2_billing_license_fee_updated_event",
        "V2BillingLicenseFeeUpdatedEventNotification",
    ),
    "v2.billing.license_fee_version.created": (
        "stripe.events._v2_billing_license_fee_version_created_event",
        "V2BillingLicenseFeeVersionCreatedEventNotification",
    ),
    "v2.billing.metered_item.created": (
        "stripe.events._v2_billing_metered_item_created_event",
        "V2BillingMeteredItemCreatedEventNotification",
    ),
    "v2.billing.metered_item.updated": (
        "stripe.events._v2_billing_metered_item_updated_event",
        "V2BillingMeteredItemUpdatedEventNotification",
    ),
    "v2.billing.pricing_plan_component.created": (
        "stripe.events._v2_billing_pricing_plan_component_created_event",
        "V2BillingPricingPlanComponentCreatedEventNotification",
    ),
    "v2.billing.pricing_plan_component.updated": (
        "stripe.events._v2_billing_pricing_plan_component_updated_event",
        "V2BillingPricingPlanComponentUpdatedEventNotification",
    ),
    "v2.billing.pricing_plan.created": (
        "stripe.events._v2_billing_pricing_plan_created_event",
        "V2BillingPricingPlanCreatedEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.collection_awaiting_customer_action": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_awaiting_customer_action_event",
        "V2BillingPricingPlanSubscriptionCollectionAwaitingCustomerActionEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.collection_current": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_current_event",
        "V2BillingPricingPlanSubscriptionCollectionCurrentEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.collection_past_due": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_past_due_event",
        "V2BillingPricingPlanSubscriptionCollectionPastDueEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.collection_paused": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_paused_event",
        "V2BillingPricingPlanSubscriptionCollectionPausedEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.collection_unpaid": (
        "stripe.events._v2_billing_pricing_plan_subscription_collection_unpaid_event",
        "V2BillingPricingPlanSubscriptionCollectionUnpaidEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.servicing_activated": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_activated_event",
        "V2BillingPricingPlanSubscriptionServicingActivatedEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.servicing_canceled": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_canceled_event",
        "V2BillingPricingPlanSubscriptionServicingCanceledEventNotification",
    ),
    "v2.billing.pricing_plan_subscription.servicing_paused": (
        "stripe.events._v2_billing_pricing_plan_subscription_servicing_paused_event",
        "V2BillingPricingPlanSubscriptionServicingPausedEventNotification",
    ),
    "v2.billing.pricing_plan.updated": (
        "stripe.events._v2_billing_pricing_plan_updated_event",
        "V2BillingPricingPlanUpdatedEventNotification",
    ),
    "v2.billing.pricing_plan_version.created": (
        "stripe.events._v2_billing_pricing_plan_version_created_event",
        "V2BillingPricingPlanVersionCreatedEventNotification",
    ),
    "v2.billing.rate_card.created": (
        "stripe.events._v2_billing_rate_card_created_event",
        "V2BillingRateCardCreatedEventNotification",
    ),
    "v2.billing.rate_card_rate.created": (
        "stripe.events._v2_billing_rate_card_rate_created_event",
        "V2BillingRateCardRateCreatedEventNotification",
    ),
    "v2.billing.rate_card_subscription.activated": (
        "stripe.events._v2_billing_rate_card_subscription_activated_event",
        "V2BillingRateCardSubscriptionActivatedEventNotification",
    ),
    "v2.billing.rate_card_subscription.canceled": (
        "stripe.events._v2_billing_rate_card_subscription_canceled_event",
        "V2BillingRateCardSubscriptionCanceledEventNotification",
    ),
    "v2.billing.rate_card_subscription.collection_awaiting_customer_action": (
        "stripe.events._v2_billing_rate_card_subscription_collection_awaiting_customer_action_event",
        "V2BillingRateCardSubscriptionCollectionAwaitingCustomerActionEventNotification",
    ),
    "v2.billing.rate_card_subscription.collection_current": (
        "stripe.events._v2_billing_rate_card_subscription_collection_current_event",
        "V2BillingRateCardSubscriptionCollectionCurrentEventNotification",
    ),
    "v2.billing.rate_card_subscription.collection_past_due": (
        "stripe.events._v2_billing_rate_card_subscription_collection_past_due_event",
        "V2BillingRateCardSubscriptionCollectionPastDueEventNotification",
    ),
    "v2.billing.rate_card_subscription.collection_paused": (
        "stripe.events._v2_billing_rate_card_subscription_collection_paused_event",
        "V2BillingRateCardSubscriptionCollectionPausedEventNotification",
    ),
    "v2.billing.rate_card_subscription.collection_unpaid": (
        "stripe.events._v2_billing_rate_card_subscription_collection_unpaid_event",
        "V2BillingRateCardSubscriptionCollectionUnpaidEventNotification",
    ),
    "v2.billing.rate_card_subscription.servicing_activated": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_activated_event",
        "V2BillingRateCardSubscriptionServicingActivatedEventNotification",
    ),
    "v2.billing.rate_card_subscription.servicing_canceled": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_canceled_event",
        "V2BillingRateCardSubscriptionServicingCanceledEventNotification",
    ),
    "v2.billing.rate_card_subscription.servicing_paused": (
        "stripe.events._v2_billing_rate_card_subscription_servicing_paused_event",
        "V2BillingRateCardSubscriptionServicingPausedEventNotification",
    ),
    "v2.billing.rate_card.updated": (
        "stripe.events._v2_billing_rate_card_updated_event",
        "V2BillingRateCardUpdatedEventNotification",
    ),
    "v2.billing.rate_card_version.created": (
        "stripe.events._v2_billing_rate_card_version_created_event",
        "V2BillingRateCardVersionCreatedEventNotification",
    ),
    "v2.core.account.closed": (
        "stripe.events._v2_core_account_closed_event",
        "V2CoreAccountClosedEventNotification",
    ),
    "v2.core.account.created": (
        "stripe.events._v2_core_account_created_event",
        "V2CoreAccountCreatedEventNotification",
    ),
    "v2.core.account[configuration.card_creator].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_card_creator_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationCardCreatorCapabilityStatusUpdatedEventNotification",
    ),
    "v2.core.account[configuration.card_creator].updated": (
        "stripe.events._v2_core_account_including_configuration_card_creator_updated_event",
        "V2CoreAccountIncludingConfigurationCardCreatorUpdatedEventNotification",
    ),
    "v2.core.account[configuration.customer].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_customer_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEventNotification",
    ),
    "v2.core.account[configuration.customer].updated": (
        "stripe.events._v2_core_account_including_configuration_customer_updated_event",
        "V2CoreAccountIncludingConfigurationCustomerUpdatedEventNotification",
    ),
    "v2.core.account[configuration.merchant].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_merchant_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventNotification",
    ),
    "v2.core.account[configuration.merchant].updated": (
        "stripe.events._v2_core_account_including_configuration_merchant_updated_event",
        "V2CoreAccountIncludingConfigurationMerchantUpdatedEventNotification",
    ),
    "v2.core.account[configuration.recipient].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_recipient_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEventNotification",
    ),
    "v2.core.account[configuration.recipient].updated": (
        "stripe.events._v2_core_account_including_configuration_recipient_updated_event",
        "V2CoreAccountIncludingConfigurationRecipientUpdatedEventNotification",
    ),
    "v2.core.account[configuration.storer].capability_status_updated": (
        "stripe.events._v2_core_account_including_configuration_storer_capability_status_updated_event",
        "V2CoreAccountIncludingConfigurationStorerCapabilityStatusUpdatedEventNotification",
    ),
    "v2.core.account[configuration.storer].updated": (
        "stripe.events._v2_core_account_including_configuration_storer_updated_event",
        "V2CoreAccountIncludingConfigurationStorerUpdatedEventNotification",
    ),
    "v2.core.account[defaults].updated": (
        "stripe.events._v2_core_account_including_defaults_updated_event",
        "V2CoreAccountIncludingDefaultsUpdatedEventNotification",
    ),
    "v2.core.account[identity].updated": (
        "stripe.events._v2_core_account_including_identity_updated_event",
        "V2CoreAccountIncludingIdentityUpdatedEventNotification",
    ),
    "v2.core.account[requirements].updated": (
        "stripe.events._v2_core_account_including_requirements_updated_event",
        "V2CoreAccountIncludingRequirementsUpdatedEventNotification",
    ),
    "v2.core.account_link.returned": (
        "stripe.events._v2_core_account_link_returned_event",
        "V2CoreAccountLinkReturnedEventNotification",
    ),
    "v2.core.account_person.created": (
        "stripe.events._v2_core_account_person_created_event",
        "V2CoreAccountPersonCreatedEventNotification",
    ),
    "v2.core.account_person.deleted": (
        "stripe.events._v2_core_account_person_deleted_event",
        "V2CoreAccountPersonDeletedEventNotification",
    ),
    "v2.core.account_person.updated": (
        "stripe.events._v2_core_account_person_updated_event",
        "V2CoreAccountPersonUpdatedEventNotification",
    ),
    "v2.core.account.updated": (
        "stripe.events._v2_core_account_updated_event",
        "V2CoreAccountUpdatedEventNotification",
    ),
    "v2.core.claimable_sandbox.claimed": (
        "stripe.events._v2_core_claimable_sandbox_claimed_event",
        "V2CoreClaimableSandboxClaimedEventNotification",
    ),
    "v2.core.claimable_sandbox.created": (
        "stripe.events._v2_core_claimable_sandbox_created_event",
        "V2CoreClaimableSandboxCreatedEventNotification",
    ),
    "v2.core.claimable_sandbox.expired": (
        "stripe.events._v2_core_claimable_sandbox_expired_event",
        "V2CoreClaimableSandboxExpiredEventNotification",
    ),
    "v2.core.claimable_sandbox.expiring": (
        "stripe.events._v2_core_claimable_sandbox_expiring_event",
        "V2CoreClaimableSandboxExpiringEventNotification",
    ),
    "v2.core.claimable_sandbox.sandbox_details_owner_account_updated": (
        "stripe.events._v2_core_claimable_sandbox_sandbox_details_owner_account_updated_event",
        "V2CoreClaimableSandboxSandboxDetailsOwnerAccountUpdatedEventNotification",
    ),
    "v2.core.event_destination.ping": (
        "stripe.events._v2_core_event_destination_ping_event",
        "V2CoreEventDestinationPingEventNotification",
    ),
    "v2.core.health.api_error.firing": (
        "stripe.events._v2_core_health_api_error_firing_event",
        "V2CoreHealthApiErrorFiringEventNotification",
    ),
    "v2.core.health.api_error.resolved": (
        "stripe.events._v2_core_health_api_error_resolved_event",
        "V2CoreHealthApiErrorResolvedEventNotification",
    ),
    "v2.core.health.api_latency.firing": (
        "stripe.events._v2_core_health_api_latency_firing_event",
        "V2CoreHealthApiLatencyFiringEventNotification",
    ),
    "v2.core.health.api_latency.resolved": (
        "stripe.events._v2_core_health_api_latency_resolved_event",
        "V2CoreHealthApiLatencyResolvedEventNotification",
    ),
    "v2.core.health.authorization_rate_drop.firing": (
        "stripe.events._v2_core_health_authorization_rate_drop_firing_event",
        "V2CoreHealthAuthorizationRateDropFiringEventNotification",
    ),
    "v2.core.health.authorization_rate_drop.resolved": (
        "stripe.events._v2_core_health_authorization_rate_drop_resolved_event",
        "V2CoreHealthAuthorizationRateDropResolvedEventNotification",
    ),
    "v2.core.health.event_generation_failure.resolved": (
        "stripe.events._v2_core_health_event_generation_failure_resolved_event",
        "V2CoreHealthEventGenerationFailureResolvedEventNotification",
    ),
    "v2.core.health.fraud_rate.increased": (
        "stripe.events._v2_core_health_fraud_rate_increased_event",
        "V2CoreHealthFraudRateIncreasedEventNotification",
    ),
    "v2.core.health.issuing_authorization_request_errors.firing": (
        "stripe.events._v2_core_health_issuing_authorization_request_errors_firing_event",
        "V2CoreHealthIssuingAuthorizationRequestErrorsFiringEventNotification",
    ),
    "v2.core.health.issuing_authorization_request_errors.resolved": (
        "stripe.events._v2_core_health_issuing_authorization_request_errors_resolved_event",
        "V2CoreHealthIssuingAuthorizationRequestErrorsResolvedEventNotification",
    ),
    "v2.core.health.issuing_authorization_request_timeout.firing": (
        "stripe.events._v2_core_health_issuing_authorization_request_timeout_firing_event",
        "V2CoreHealthIssuingAuthorizationRequestTimeoutFiringEventNotification",
    ),
    "v2.core.health.issuing_authorization_request_timeout.resolved": (
        "stripe.events._v2_core_health_issuing_authorization_request_timeout_resolved_event",
        "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventNotification",
    ),
    "v2.core.health.payment_method_error.firing": (
        "stripe.events._v2_core_health_payment_method_error_firing_event",
        "V2CoreHealthPaymentMethodErrorFiringEventNotification",
    ),
    "v2.core.health.payment_method_error.resolved": (
        "stripe.events._v2_core_health_payment_method_error_resolved_event",
        "V2CoreHealthPaymentMethodErrorResolvedEventNotification",
    ),
    "v2.core.health.traffic_volume_drop.firing": (
        "stripe.events._v2_core_health_traffic_volume_drop_firing_event",
        "V2CoreHealthTrafficVolumeDropFiringEventNotification",
    ),
    "v2.core.health.traffic_volume_drop.resolved": (
        "stripe.events._v2_core_health_traffic_volume_drop_resolved_event",
        "V2CoreHealthTrafficVolumeDropResolvedEventNotification",
    ),
    "v2.core.health.webhook_latency.firing": (
        "stripe.events._v2_core_health_webhook_latency_firing_event",
        "V2CoreHealthWebhookLatencyFiringEventNotification",
    ),
    "v2.core.health.webhook_latency.resolved": (
        "stripe.events._v2_core_health_webhook_latency_resolved_event",
        "V2CoreHealthWebhookLatencyResolvedEventNotification",
    ),
    "v2.iam.api_key.created": (
        "stripe.events._v2_iam_api_key_created_event",
        "V2IamApiKeyCreatedEventNotification",
    ),
    "v2.iam.api_key.default_secret_revealed": (
        "stripe.events._v2_iam_api_key_default_secret_revealed_event",
        "V2IamApiKeyDefaultSecretRevealedEventNotification",
    ),
    "v2.iam.api_key.expired": (
        "stripe.events._v2_iam_api_key_expired_event",
        "V2IamApiKeyExpiredEventNotification",
    ),
    "v2.iam.api_key.permissions_updated": (
        "stripe.events._v2_iam_api_key_permissions_updated_event",
        "V2IamApiKeyPermissionsUpdatedEventNotification",
    ),
    "v2.iam.api_key.rotated": (
        "stripe.events._v2_iam_api_key_rotated_event",
        "V2IamApiKeyRotatedEventNotification",
    ),
    "v2.iam.api_key.updated": (
        "stripe.events._v2_iam_api_key_updated_event",
        "V2IamApiKeyUpdatedEventNotification",
    ),
    "v2.money_management.adjustment.created": (
        "stripe.events._v2_money_management_adjustment_created_event",
        "V2MoneyManagementAdjustmentCreatedEventNotification",
    ),
    "v2.money_management.financial_account.created": (
        "stripe.events._v2_money_management_financial_account_created_event",
        "V2MoneyManagementFinancialAccountCreatedEventNotification",
    ),
    "v2.money_management.financial_account.updated": (
        "stripe.events._v2_money_management_financial_account_updated_event",
        "V2MoneyManagementFinancialAccountUpdatedEventNotification",
    ),
    "v2.money_management.financial_address.activated": (
        "stripe.events._v2_money_management_financial_address_activated_event",
        "V2MoneyManagementFinancialAddressActivatedEventNotification",
    ),
    "v2.money_management.financial_address.failed": (
        "stripe.events._v2_money_management_financial_address_failed_event",
        "V2MoneyManagementFinancialAddressFailedEventNotification",
    ),
    "v2.money_management.inbound_transfer.available": (
        "stripe.events._v2_money_management_inbound_transfer_available_event",
        "V2MoneyManagementInboundTransferAvailableEventNotification",
    ),
    "v2.money_management.inbound_transfer.bank_debit_failed": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_failed_event",
        "V2MoneyManagementInboundTransferBankDebitFailedEventNotification",
    ),
    "v2.money_management.inbound_transfer.bank_debit_processing": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_processing_event",
        "V2MoneyManagementInboundTransferBankDebitProcessingEventNotification",
    ),
    "v2.money_management.inbound_transfer.bank_debit_queued": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_queued_event",
        "V2MoneyManagementInboundTransferBankDebitQueuedEventNotification",
    ),
    "v2.money_management.inbound_transfer.bank_debit_returned": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_returned_event",
        "V2MoneyManagementInboundTransferBankDebitReturnedEventNotification",
    ),
    "v2.money_management.inbound_transfer.bank_debit_succeeded": (
        "stripe.events._v2_money_management_inbound_transfer_bank_debit_succeeded_event",
        "V2MoneyManagementInboundTransferBankDebitSucceededEventNotification",
    ),
    "v2.money_management.outbound_payment.canceled": (
        "stripe.events._v2_money_management_outbound_payment_canceled_event",
        "V2MoneyManagementOutboundPaymentCanceledEventNotification",
    ),
    "v2.money_management.outbound_payment.created": (
        "stripe.events._v2_money_management_outbound_payment_created_event",
        "V2MoneyManagementOutboundPaymentCreatedEventNotification",
    ),
    "v2.money_management.outbound_payment.failed": (
        "stripe.events._v2_money_management_outbound_payment_failed_event",
        "V2MoneyManagementOutboundPaymentFailedEventNotification",
    ),
    "v2.money_management.outbound_payment.posted": (
        "stripe.events._v2_money_management_outbound_payment_posted_event",
        "V2MoneyManagementOutboundPaymentPostedEventNotification",
    ),
    "v2.money_management.outbound_payment.returned": (
        "stripe.events._v2_money_management_outbound_payment_returned_event",
        "V2MoneyManagementOutboundPaymentReturnedEventNotification",
    ),
    "v2.money_management.outbound_payment.updated": (
        "stripe.events._v2_money_management_outbound_payment_updated_event",
        "V2MoneyManagementOutboundPaymentUpdatedEventNotification",
    ),
    "v2.money_management.outbound_transfer.canceled": (
        "stripe.events._v2_money_management_outbound_transfer_canceled_event",
        "V2MoneyManagementOutboundTransferCanceledEventNotification",
    ),
    "v2.money_management.outbound_transfer.created": (
        "stripe.events._v2_money_management_outbound_transfer_created_event",
        "V2MoneyManagementOutboundTransferCreatedEventNotification",
    ),
    "v2.money_management.outbound_transfer.failed": (
        "stripe.events._v2_money_management_outbound_transfer_failed_event",
        "V2MoneyManagementOutboundTransferFailedEventNotification",
    ),
    "v2.money_management.outbound_transfer.posted": (
        "stripe.events._v2_money_management_outbound_transfer_posted_event",
        "V2MoneyManagementOutboundTransferPostedEventNotification",
    ),
    "v2.money_management.outbound_transfer.returned": (
        "stripe.events._v2_money_management_outbound_transfer_returned_event",
        "V2MoneyManagementOutboundTransferReturnedEventNotification",
    ),
    "v2.money_management.outbound_transfer.updated": (
        "stripe.events._v2_money_management_outbound_transfer_updated_event",
        "V2MoneyManagementOutboundTransferUpdatedEventNotification",
    ),
    "v2.money_management.payout_method.updated": (
        "stripe.events._v2_money_management_payout_method_updated_event",
        "V2MoneyManagementPayoutMethodUpdatedEventNotification",
    ),
    "v2.money_management.received_credit.available": (
        "stripe.events._v2_money_management_received_credit_available_event",
        "V2MoneyManagementReceivedCreditAvailableEventNotification",
    ),
    "v2.money_management.received_credit.failed": (
        "stripe.events._v2_money_management_received_credit_failed_event",
        "V2MoneyManagementReceivedCreditFailedEventNotification",
    ),
    "v2.money_management.received_credit.returned": (
        "stripe.events._v2_money_management_received_credit_returned_event",
        "V2MoneyManagementReceivedCreditReturnedEventNotification",
    ),
    "v2.money_management.received_credit.succeeded": (
        "stripe.events._v2_money_management_received_credit_succeeded_event",
        "V2MoneyManagementReceivedCreditSucceededEventNotification",
    ),
    "v2.money_management.received_debit.canceled": (
        "stripe.events._v2_money_management_received_debit_canceled_event",
        "V2MoneyManagementReceivedDebitCanceledEventNotification",
    ),
    "v2.money_management.received_debit.failed": (
        "stripe.events._v2_money_management_received_debit_failed_event",
        "V2MoneyManagementReceivedDebitFailedEventNotification",
    ),
    "v2.money_management.received_debit.pending": (
        "stripe.events._v2_money_management_received_debit_pending_event",
        "V2MoneyManagementReceivedDebitPendingEventNotification",
    ),
    "v2.money_management.received_debit.succeeded": (
        "stripe.events._v2_money_management_received_debit_succeeded_event",
        "V2MoneyManagementReceivedDebitSucceededEventNotification",
    ),
    "v2.money_management.received_debit.updated": (
        "stripe.events._v2_money_management_received_debit_updated_event",
        "V2MoneyManagementReceivedDebitUpdatedEventNotification",
    ),
    "v2.money_management.recipient_verification.created": (
        "stripe.events._v2_money_management_recipient_verification_created_event",
        "V2MoneyManagementRecipientVerificationCreatedEventNotification",
    ),
    "v2.money_management.recipient_verification.updated": (
        "stripe.events._v2_money_management_recipient_verification_updated_event",
        "V2MoneyManagementRecipientVerificationUpdatedEventNotification",
    ),
    "v2.money_management.transaction.created": (
        "stripe.events._v2_money_management_transaction_created_event",
        "V2MoneyManagementTransactionCreatedEventNotification",
    ),
    "v2.money_management.transaction.updated": (
        "stripe.events._v2_money_management_transaction_updated_event",
        "V2MoneyManagementTransactionUpdatedEventNotification",
    ),
    "v2.payments.off_session_payment.attempt_failed": (
        "stripe.events._v2_payments_off_session_payment_attempt_failed_event",
        "V2PaymentsOffSessionPaymentAttemptFailedEventNotification",
    ),
    "v2.payments.off_session_payment.attempt_started": (
        "stripe.events._v2_payments_off_session_payment_attempt_started_event",
        "V2PaymentsOffSessionPaymentAttemptStartedEventNotification",
    ),
    "v2.payments.off_session_payment.authorization_attempt_failed": (
        "stripe.events._v2_payments_off_session_payment_authorization_attempt_failed_event",
        "V2PaymentsOffSessionPaymentAuthorizationAttemptFailedEventNotification",
    ),
    "v2.payments.off_session_payment.authorization_attempt_started": (
        "stripe.events._v2_payments_off_session_payment_authorization_attempt_started_event",
        "V2PaymentsOffSessionPaymentAuthorizationAttemptStartedEventNotification",
    ),
    "v2.payments.off_session_payment.canceled": (
        "stripe.events._v2_payments_off_session_payment_canceled_event",
        "V2PaymentsOffSessionPaymentCanceledEventNotification",
    ),
    "v2.payments.off_session_payment.created": (
        "stripe.events._v2_payments_off_session_payment_created_event",
        "V2PaymentsOffSessionPaymentCreatedEventNotification",
    ),
    "v2.payments.off_session_payment.failed": (
        "stripe.events._v2_payments_off_session_payment_failed_event",
        "V2PaymentsOffSessionPaymentFailedEventNotification",
    ),
    "v2.payments.off_session_payment.requires_capture": (
        "stripe.events._v2_payments_off_session_payment_requires_capture_event",
        "V2PaymentsOffSessionPaymentRequiresCaptureEventNotification",
    ),
    "v2.payments.off_session_payment.succeeded": (
        "stripe.events._v2_payments_off_session_payment_succeeded_event",
        "V2PaymentsOffSessionPaymentSucceededEventNotification",
    ),
    "v2.reporting.report_run.created": (
        "stripe.events._v2_reporting_report_run_created_event",
        "V2ReportingReportRunCreatedEventNotification",
    ),
    "v2.reporting.report_run.failed": (
        "stripe.events._v2_reporting_report_run_failed_event",
        "V2ReportingReportRunFailedEventNotification",
    ),
    "v2.reporting.report_run.succeeded": (
        "stripe.events._v2_reporting_report_run_succeeded_event",
        "V2ReportingReportRunSucceededEventNotification",
    ),
    "v2.reporting.report_run.updated": (
        "stripe.events._v2_reporting_report_run_updated_event",
        "V2ReportingReportRunUpdatedEventNotification",
    ),
}


def get_v2_event_notification_class(type_: str):
    if type_ not in _V2_EVENT_NOTIFICATION_CLASS_LOOKUP:
        return UnknownEventNotification

    import_path, class_name = _V2_EVENT_NOTIFICATION_CLASS_LOOKUP[type_]
    return getattr(
        import_module(import_path),
        class_name,
    )


ALL_EVENT_NOTIFICATIONS = Union[
    "V1BillingMeterErrorReportTriggeredEventNotification",
    "V1BillingMeterNoMeterFoundEventNotification",
    "V2BillingCadenceBilledEventNotification",
    "V2BillingCadenceCanceledEventNotification",
    "V2BillingCadenceCreatedEventNotification",
    "V2BillingLicensedItemCreatedEventNotification",
    "V2BillingLicensedItemUpdatedEventNotification",
    "V2BillingLicenseFeeCreatedEventNotification",
    "V2BillingLicenseFeeUpdatedEventNotification",
    "V2BillingLicenseFeeVersionCreatedEventNotification",
    "V2BillingMeteredItemCreatedEventNotification",
    "V2BillingMeteredItemUpdatedEventNotification",
    "V2BillingPricingPlanComponentCreatedEventNotification",
    "V2BillingPricingPlanComponentUpdatedEventNotification",
    "V2BillingPricingPlanCreatedEventNotification",
    "V2BillingPricingPlanSubscriptionCollectionAwaitingCustomerActionEventNotification",
    "V2BillingPricingPlanSubscriptionCollectionCurrentEventNotification",
    "V2BillingPricingPlanSubscriptionCollectionPastDueEventNotification",
    "V2BillingPricingPlanSubscriptionCollectionPausedEventNotification",
    "V2BillingPricingPlanSubscriptionCollectionUnpaidEventNotification",
    "V2BillingPricingPlanSubscriptionServicingActivatedEventNotification",
    "V2BillingPricingPlanSubscriptionServicingCanceledEventNotification",
    "V2BillingPricingPlanSubscriptionServicingPausedEventNotification",
    "V2BillingPricingPlanUpdatedEventNotification",
    "V2BillingPricingPlanVersionCreatedEventNotification",
    "V2BillingRateCardCreatedEventNotification",
    "V2BillingRateCardRateCreatedEventNotification",
    "V2BillingRateCardSubscriptionActivatedEventNotification",
    "V2BillingRateCardSubscriptionCanceledEventNotification",
    "V2BillingRateCardSubscriptionCollectionAwaitingCustomerActionEventNotification",
    "V2BillingRateCardSubscriptionCollectionCurrentEventNotification",
    "V2BillingRateCardSubscriptionCollectionPastDueEventNotification",
    "V2BillingRateCardSubscriptionCollectionPausedEventNotification",
    "V2BillingRateCardSubscriptionCollectionUnpaidEventNotification",
    "V2BillingRateCardSubscriptionServicingActivatedEventNotification",
    "V2BillingRateCardSubscriptionServicingCanceledEventNotification",
    "V2BillingRateCardSubscriptionServicingPausedEventNotification",
    "V2BillingRateCardUpdatedEventNotification",
    "V2BillingRateCardVersionCreatedEventNotification",
    "V2CoreAccountClosedEventNotification",
    "V2CoreAccountCreatedEventNotification",
    "V2CoreAccountIncludingConfigurationCardCreatorCapabilityStatusUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationCardCreatorUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationCustomerUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationMerchantUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationRecipientUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationStorerCapabilityStatusUpdatedEventNotification",
    "V2CoreAccountIncludingConfigurationStorerUpdatedEventNotification",
    "V2CoreAccountIncludingDefaultsUpdatedEventNotification",
    "V2CoreAccountIncludingIdentityUpdatedEventNotification",
    "V2CoreAccountIncludingRequirementsUpdatedEventNotification",
    "V2CoreAccountLinkReturnedEventNotification",
    "V2CoreAccountPersonCreatedEventNotification",
    "V2CoreAccountPersonDeletedEventNotification",
    "V2CoreAccountPersonUpdatedEventNotification",
    "V2CoreAccountUpdatedEventNotification",
    "V2CoreClaimableSandboxClaimedEventNotification",
    "V2CoreClaimableSandboxCreatedEventNotification",
    "V2CoreClaimableSandboxExpiredEventNotification",
    "V2CoreClaimableSandboxExpiringEventNotification",
    "V2CoreClaimableSandboxSandboxDetailsOwnerAccountUpdatedEventNotification",
    "V2CoreEventDestinationPingEventNotification",
    "V2CoreHealthApiErrorFiringEventNotification",
    "V2CoreHealthApiErrorResolvedEventNotification",
    "V2CoreHealthApiLatencyFiringEventNotification",
    "V2CoreHealthApiLatencyResolvedEventNotification",
    "V2CoreHealthAuthorizationRateDropFiringEventNotification",
    "V2CoreHealthAuthorizationRateDropResolvedEventNotification",
    "V2CoreHealthEventGenerationFailureResolvedEventNotification",
    "V2CoreHealthFraudRateIncreasedEventNotification",
    "V2CoreHealthIssuingAuthorizationRequestErrorsFiringEventNotification",
    "V2CoreHealthIssuingAuthorizationRequestErrorsResolvedEventNotification",
    "V2CoreHealthIssuingAuthorizationRequestTimeoutFiringEventNotification",
    "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventNotification",
    "V2CoreHealthPaymentMethodErrorFiringEventNotification",
    "V2CoreHealthPaymentMethodErrorResolvedEventNotification",
    "V2CoreHealthTrafficVolumeDropFiringEventNotification",
    "V2CoreHealthTrafficVolumeDropResolvedEventNotification",
    "V2CoreHealthWebhookLatencyFiringEventNotification",
    "V2CoreHealthWebhookLatencyResolvedEventNotification",
    "V2IamApiKeyCreatedEventNotification",
    "V2IamApiKeyDefaultSecretRevealedEventNotification",
    "V2IamApiKeyExpiredEventNotification",
    "V2IamApiKeyPermissionsUpdatedEventNotification",
    "V2IamApiKeyRotatedEventNotification",
    "V2IamApiKeyUpdatedEventNotification",
    "V2MoneyManagementAdjustmentCreatedEventNotification",
    "V2MoneyManagementFinancialAccountCreatedEventNotification",
    "V2MoneyManagementFinancialAccountUpdatedEventNotification",
    "V2MoneyManagementFinancialAddressActivatedEventNotification",
    "V2MoneyManagementFinancialAddressFailedEventNotification",
    "V2MoneyManagementInboundTransferAvailableEventNotification",
    "V2MoneyManagementInboundTransferBankDebitFailedEventNotification",
    "V2MoneyManagementInboundTransferBankDebitProcessingEventNotification",
    "V2MoneyManagementInboundTransferBankDebitQueuedEventNotification",
    "V2MoneyManagementInboundTransferBankDebitReturnedEventNotification",
    "V2MoneyManagementInboundTransferBankDebitSucceededEventNotification",
    "V2MoneyManagementOutboundPaymentCanceledEventNotification",
    "V2MoneyManagementOutboundPaymentCreatedEventNotification",
    "V2MoneyManagementOutboundPaymentFailedEventNotification",
    "V2MoneyManagementOutboundPaymentPostedEventNotification",
    "V2MoneyManagementOutboundPaymentReturnedEventNotification",
    "V2MoneyManagementOutboundPaymentUpdatedEventNotification",
    "V2MoneyManagementOutboundTransferCanceledEventNotification",
    "V2MoneyManagementOutboundTransferCreatedEventNotification",
    "V2MoneyManagementOutboundTransferFailedEventNotification",
    "V2MoneyManagementOutboundTransferPostedEventNotification",
    "V2MoneyManagementOutboundTransferReturnedEventNotification",
    "V2MoneyManagementOutboundTransferUpdatedEventNotification",
    "V2MoneyManagementPayoutMethodUpdatedEventNotification",
    "V2MoneyManagementReceivedCreditAvailableEventNotification",
    "V2MoneyManagementReceivedCreditFailedEventNotification",
    "V2MoneyManagementReceivedCreditReturnedEventNotification",
    "V2MoneyManagementReceivedCreditSucceededEventNotification",
    "V2MoneyManagementReceivedDebitCanceledEventNotification",
    "V2MoneyManagementReceivedDebitFailedEventNotification",
    "V2MoneyManagementReceivedDebitPendingEventNotification",
    "V2MoneyManagementReceivedDebitSucceededEventNotification",
    "V2MoneyManagementReceivedDebitUpdatedEventNotification",
    "V2MoneyManagementRecipientVerificationCreatedEventNotification",
    "V2MoneyManagementRecipientVerificationUpdatedEventNotification",
    "V2MoneyManagementTransactionCreatedEventNotification",
    "V2MoneyManagementTransactionUpdatedEventNotification",
    "V2PaymentsOffSessionPaymentAttemptFailedEventNotification",
    "V2PaymentsOffSessionPaymentAttemptStartedEventNotification",
    "V2PaymentsOffSessionPaymentAuthorizationAttemptFailedEventNotification",
    "V2PaymentsOffSessionPaymentAuthorizationAttemptStartedEventNotification",
    "V2PaymentsOffSessionPaymentCanceledEventNotification",
    "V2PaymentsOffSessionPaymentCreatedEventNotification",
    "V2PaymentsOffSessionPaymentFailedEventNotification",
    "V2PaymentsOffSessionPaymentRequiresCaptureEventNotification",
    "V2PaymentsOffSessionPaymentSucceededEventNotification",
    "V2ReportingReportRunCreatedEventNotification",
    "V2ReportingReportRunFailedEventNotification",
    "V2ReportingReportRunSucceededEventNotification",
    "V2ReportingReportRunUpdatedEventNotification",
]



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class PlanCreateParams(RequestOptions):
    active: NotRequired[bool]
    """
    Whether the plan is currently available for new subscriptions. Defaults to `true`.
    """
    amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free plan) representing how much to charge on a recurring basis.
    """
    amount_decimal: NotRequired[str]
    """
    Same as `amount`, but accepts a decimal value with at most 12 decimal places. Only one of `amount` and `amount_decimal` can be set.
    """
    billing_scheme: NotRequired[Literal["per_unit", "tiered"]]
    """
    Describes how to compute the price per period. Either `per_unit` or `tiered`. `per_unit` indicates that the fixed amount (specified in `amount`) will be charged per unit in `quantity` (for plans with `usage_type=licensed`), or per unit of total usage (for plans with `usage_type=metered`). `tiered` indicates that the unit pricing will be computed using a tiering strategy as defined using the `tiers` and `tiers_mode` attributes.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    id: NotRequired[str]
    """
    An identifier randomly generated by Stripe. Used to identify this plan when subscribing a customer. You can optionally override this ID, but the ID must be unique across all plans in your Stripe account. You can, however, use the same plan ID in both live and test modes.
    """
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing frequency. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between subscription billings. For example, `interval=month` and `interval_count=3` bills every 3 months. Maximum of three years interval allowed (3 years, 36 months, or 156 weeks).
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    meter: NotRequired[str]
    """
    The meter tracking the usage of a metered price
    """
    nickname: NotRequired[str]
    """
    A brief description of the plan, hidden from customers.
    """
    product: NotRequired["PlanCreateParamsProduct|str"]
    tiers: NotRequired[List["PlanCreateParamsTier"]]
    """
    Each element represents a pricing tier. This parameter requires `billing_scheme` to be set to `tiered`. See also the documentation for `billing_scheme`.
    """
    tiers_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines if the tiering price should be `graduated` or `volume` based. In `volume`-based tiering, the maximum quantity within a period determines the per unit price, in `graduated` tiering pricing can successively change as the quantity grows.
    """
    transform_usage: NotRequired["PlanCreateParamsTransformUsage"]
    """
    Apply a transformation to the reported usage or set quantity before computing the billed price. Cannot be combined with `tiers`.
    """
    trial_period_days: NotRequired[int]
    """
    Default number of trial days when subscribing a customer to this plan using [`trial_from_plan=true`](https://stripe.com/docs/api#create_subscription-trial_from_plan).
    """
    usage_type: NotRequired[Literal["licensed", "metered"]]
    """
    Configures how the quantity per period should be determined. Can be either `metered` or `licensed`. `licensed` automatically bills the `quantity` set when adding it to a subscription. `metered` aggregates the total usage based on usage records. Defaults to `licensed`.
    """


class PlanCreateParamsProduct(TypedDict):
    active: NotRequired[bool]
    """
    Whether the product is currently available for purchase. Defaults to `true`.
    """
    id: NotRequired[str]
    """
    The identifier for the product. Must be unique. If not provided, an identifier will be randomly generated.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: str
    """
    The product's name, meant to be displayable to the customer.
    """
    statement_descriptor: NotRequired[str]
    """
    An arbitrary string to be displayed on your customer's credit card or bank statement. While most banks display this information consistently, some may display it incorrectly or not at all.

    This may be up to 22 characters. The statement description may not include `<`, `>`, `\\`, `"`, `'` characters, and will appear on your customer's statement in capital letters. Non-ASCII characters are automatically stripped.
    """
    tax_code: NotRequired[str]
    """
    A [tax code](https://stripe.com/docs/tax/tax-categories) ID.
    """
    unit_label: NotRequired[str]
    """
    A label that represents units of this product. When set, this will be included in customers' receipts, invoices, Checkout, and the customer portal.
    """


class PlanCreateParamsTier(TypedDict):
    flat_amount: NotRequired[int]
    """
    The flat billing amount for an entire tier, regardless of the number of units in the tier.
    """
    flat_amount_decimal: NotRequired[str]
    """
    Same as `flat_amount`, but accepts a decimal value representing an integer in the minor units of the currency. Only one of `flat_amount` and `flat_amount_decimal` can be set.
    """
    unit_amount: NotRequired[int]
    """
    The per unit billing amount for each individual unit for which this tier applies.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """
    up_to: Union[Literal["inf"], int]
    """
    Specifies the upper bound of this tier. The lower bound of a tier is the upper bound of the previous tier adding one. Use `inf` to define a fallback tier.
    """


class PlanCreateParamsTransformUsage(TypedDict):
    divide_by: int
    """
    Divide usage by this number.
    """
    round: Literal["down", "up"]
    """
    After division, either round the result `up` or `down`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import Dict, List
from typing_extensions import NotRequired, TypedDict


class ValueListUpdateParams(TypedDict):
    alias: NotRequired[str]
    """
    The name of the value list for use in rules.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    The human-readable name of the value list.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._quote_preview_subscription_schedule import (
        QuotePreviewSubscriptionSchedule,
    )
    from stripe._request_options import RequestOptions
    from stripe.params._quote_preview_subscription_schedule_list_params import (
        QuotePreviewSubscriptionScheduleListParams,
    )


class QuotePreviewSubscriptionScheduleService(StripeService):
    def list(
        self,
        quote: str,
        params: Optional["QuotePreviewSubscriptionScheduleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[QuotePreviewSubscriptionSchedule]":
        """
        Preview the schedules that would be generated by accepting the quote
        """
        return cast(
            "ListObject[QuotePreviewSubscriptionSchedule]",
            self._request(
                "get",
                "/v1/quotes/{quote}/preview_subscription_schedules".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        quote: str,
        params: Optional["QuotePreviewSubscriptionScheduleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[QuotePreviewSubscriptionSchedule]":
        """
        Preview the schedules that would be generated by accepting the quote
        """
        return cast(
            "ListObject[QuotePreviewSubscriptionSchedule]",
            await self._request_async(
                "get",
                "/v1/quotes/{quote}/preview_subscription_schedules".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
